// This file is automatically generated. DO NOT EDIT.
// See codegen/gen_dictionary.rs for the generation code.
#![allow(clippy::unreadable_literal)]

//! The dictionary contains the information required to parse dates and numbers
//! in all supported languages.

use crate::{
    model::AlbumType,
    param::Language,
    util::timeago::{DateCmp, TaToken, TimeUnit},
};

/// Dictionary entry containing language-specific parsing information
pub(crate) struct Entry {
    /// Tokens for parsing timeago strings.
    ///
    /// Format: Parsed token -> \[Quantity\] Identifier
    ///
    /// Identifiers: `Y`(ear), `M`(month), `W`(eek), `D`(ay),
    /// `h`(our), `m`(inute), `s`(econd)
    pub timeago_tokens: phf::Map<&'static str, TaToken>,
    /// Order in which to parse numeric date components. Formatted as
    /// a string of date identifiers (Y, M, D).
    ///
    /// Examples:
    ///
    /// - 03.01.2020 => `"DMY"`
    /// - Jan 3, 2020 => `"DY"`
    pub date_order: &'static [DateCmp],
    /// Tokens for parsing month names.
    ///
    /// Format: Parsed token -> Month number (starting from 1)
    pub months: phf::Map<&'static str, u8>,
    /// Tokens for parsing date strings with no digits (e.g. Today, Tomorrow)
    ///
    /// Format: Parsed token -> \[Quantity\] Identifier
    pub timeago_nd_tokens: phf::Map<&'static str, TaToken>,
    /// Are commas (instead of points) used as decimal separators?
    pub comma_decimal: bool,
    /// Tokens for parsing decimal prefixes (K, M, B, ...)
    ///
    /// Format: Parsed token -> decimal power
    pub number_tokens: phf::Map<&'static str, u8>,
    /// Tokens for parsing number strings with no digits (e.g. "No videos")
    ///
    /// Format: Parsed token -> value
    pub number_nd_tokens: phf::Map<&'static str, u8>,
    /// Names of album types (Album, Single, ...)
    ///
    /// Format: Parsed text -> Album type
    pub album_types: phf::Map<&'static str, AlbumType>,
}

#[rustfmt::skip]
pub(crate) fn entry(lang: Language) -> Entry {
    match lang {
        Language::Af => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (4, 0),
                    (0, 9),
                    (0, 3),
                    (6, 0),
                ],
                entries: &[
                    ("dae", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("weke", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sekondes", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("j", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("week", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("md", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("uur", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dag", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("maand", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("w", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sekonde", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minute", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minuut", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("maande", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("u", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("jaar", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (4, 8),
                    (1, 0),
                    (7, 2),
                ],
                entries: &[
                    ("nov", 11),
                    ("okt", 10),
                    ("apr", 4),
                    ("jun", 6),
                    ("mrt", 3),
                    ("aug", 8),
                    ("des", 12),
                    ("mei", 5),
                    ("jan", 1),
                    ("jul", 7),
                    ("sep", 9),
                    ("feb", 2),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("gister", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("vandag", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mjd", 9),
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("nie", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 2980949210194914378,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("album", AlbumType::Album),
                    ("drama", AlbumType::Show),
                    ("oudioboek", AlbumType::Audiobook),
                    ("ep", AlbumType::Ep),
                    ("enkelsnit", AlbumType::Single),
                ],
            },
        },
        Language::Am => Entry {
            timeago_tokens: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (6, 0),
                    (8, 8),
                    (1, 15),
                    (6, 2),
                ],
                entries: &[
                    ("ቀን", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ሰከንድ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ሳምንታት", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ዓመት", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ሰዓቶች", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ወራት", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ዓመታት", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ደቂቃዎች፣", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ሳምንት", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ሰዓት", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ሰዓ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ሰከንዶች", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ወር", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ደቂቃዎች", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ሰከ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ደቂቃ", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ደቂቃ፣", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ቀኖች", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ቀናት", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (2, 0),
                    (0, 10),
                    (9, 7),
                ],
                entries: &[
                    ("ሜይ", 5),
                    ("ሴፕቴ", 9),
                    ("ዲሴም", 12),
                    ("ፌብ", 2),
                    ("ፌብሩ", 2),
                    ("ኦገስ", 8),
                    ("ማርች", 3),
                    ("ኤፕሪ", 4),
                    ("ኦክቶ", 10),
                    ("ጃንዩ", 1),
                    ("ኖቬም", 11),
                    ("ጁን", 6),
                    ("ጃን", 1),
                    ("ጁላይ", 7),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ትላንት", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ዛሬ", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ሚ", 6),
                    ("ሺ", 3),
                    ("ቢ", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("የለዉም", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (4, 0),
                ],
                entries: &[
                    ("ነጠላ", AlbumType::Single),
                    ("የተራዘመ አልበም", AlbumType::Ep),
                    ("ኦዲዮ መጽሐፍ", AlbumType::Audiobook),
                    ("ትዕይንት", AlbumType::Show),
                    ("አልበም", AlbumType::Album),
                ],
            },
        },
        Language::Ar => Entry {
            timeago_tokens: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (3, 13),
                    (11, 32),
                    (0, 0),
                    (0, 0),
                    (4, 28),
                    (0, 1),
                    (5, 12),
                ],
                entries: &[
                    ("شهر", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("وثانية", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("دقيقتان", TaToken { n: 2, unit: Some(TimeUnit::Minute) }),
                    ("ثوان\u{64d}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("يوم", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("شهر\u{64b}ا", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("سنة", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("يوم\u{64b}ا", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("دقيقة", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ثوان", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ثانية", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ساعات", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("أيام", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ساعتين", TaToken { n: 2, unit: Some(TimeUnit::Hour) }),
                    ("أسبوع\u{64b}ا", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("سنوات", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ساعة", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ثانيتين", TaToken { n: 2, unit: Some(TimeUnit::Second) }),
                    ("أسابيع", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("أشهر", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("أسبوع", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("شهرين", TaToken { n: 2, unit: Some(TimeUnit::Month) }),
                    ("وثانيتان", TaToken { n: 2, unit: Some(TimeUnit::Second) }),
                    ("يومين", TaToken { n: 2, unit: Some(TimeUnit::Day) }),
                    ("س", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ث", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("دقيقتين", TaToken { n: 2, unit: Some(TimeUnit::Minute) }),
                    ("خلال", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ثوان\u{650}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("د", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("سنتين", TaToken { n: 2, unit: Some(TimeUnit::Year) }),
                    ("أسبوعين", TaToken { n: 2, unit: Some(TimeUnit::Week) }),
                    ("دقائق", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("البارحة", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("اليوم", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("آلاف", 3),
                    ("مليون", 6),
                    ("ألف", 3),
                    ("مليار", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("واحد", 1),
                    ("لا", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ألبوم", AlbumType::Album),
                    ("ألبوم قصير", AlbumType::Ep),
                    ("أغنية منفردة", AlbumType::Single),
                    ("عرض", AlbumType::Show),
                    ("الكتب المسموعة", AlbumType::Audiobook),
                ],
            },
        },
        Language::As => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (5, 5),
                    (3, 0),
                ],
                entries: &[
                    ("দিন", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ছেকেণ\u{9cd}ড", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("বছৰৰ", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ঘণ\u{9cd}ট\u{9be}", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("সপ\u{9cd}ত\u{9be}হ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ম\u{9be}হ", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("মিনিট", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("আজি", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ক\u{9be}লি", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (1, 0),
                    (3, 6),
                    (1, 4),
                ],
                entries: &[
                    ("হ\u{9be}", 3),
                    ("কোঃ", 5),
                    ("ল\u{9be}খট\u{9be}", 5),
                    ("ল\u{9be}খ", 5),
                    ("নিঃট\u{9be}", 6),
                    ("হ\u{9be}জ\u{9be}ৰট\u{9be}", 3),
                    ("কোঃট\u{9be}", 9),
                    ("নিঃ", 6),
                    ("নিয\u{9c1}তট\u{9be}", 6),
                    ("ল\u{9be}", 5),
                    ("নিয\u{9c1}ত", 6),
                    ("হ\u{9be}জ\u{9be}ৰ", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("১", 1),
                    ("ন\u{9be}ই", 0),
                    ("১ট\u{9be}", 1),
                ],
            },
            album_types: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (4, 0),
                ],
                entries: &[
                    ("এলব\u{9be}ম", AlbumType::Album),
                    ("একক", AlbumType::Single),
                    ("অডিঅ’ব\u{9c1}ক", AlbumType::Audiobook),
                    ("শ\u{9cd}ব’", AlbumType::Show),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::Az => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (4, 0),
                    (8, 1),
                ],
                entries: &[
                    ("həftə", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("dəq", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("gün", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("saat", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dəqiqə", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("il", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("saniyə", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("san", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ay", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (1, 5),
                    (4, 0),
                    (0, 10),
                ],
                entries: &[
                    ("okt", 10),
                    ("sen", 9),
                    ("yan", 1),
                    ("mar", 3),
                    ("iyl", 7),
                    ("noy", 11),
                    ("dek", 12),
                    ("fev", 2),
                    ("iyn", 6),
                    ("avq", 8),
                    ("may", 5),
                    ("apr", 4),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("dünən", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("bugün", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mlrd", 9),
                    ("mln", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("yoxdur", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("albom", AlbumType::Album),
                    ("audio kitab", AlbumType::Audiobook),
                    ("şou", AlbumType::Show),
                    ("tək", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::Be => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (7, 12),
                    (0, 0),
                    (0, 0),
                    (5, 21),
                    (0, 3),
                    (31, 31),
                    (1, 16),
                ],
                entries: &[
                    ("гадз", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("год", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("гадзіну", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("года", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("гадзіны", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("месяца", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("дзён", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("мес", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("тыдні", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("хвіліну", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("хвілін", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("секунду", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("хв", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("секунд", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("дня", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("дзень", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("тыдняў", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("месяцаў", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("секунды", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("гады", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("секунда", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("тыдзень", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("месяц", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("гадоў", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("хвіліны", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("г", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("с", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("тыдня", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("месяцы", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("гадзін", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("тыд", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("дні", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("хвіліна", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (11, 11),
                    (2, 2),
                    (0, 0),
                ],
                entries: &[
                    ("лют", 2),
                    ("ліп", 7),
                    ("чэр", 6),
                    ("кас", 10),
                    ("сак", 3),
                    ("сту", 1),
                    ("сне", 12),
                    ("ліс", 11),
                    ("вер", 9),
                    ("кра", 4),
                    ("мая", 5),
                    ("жні", 8),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("сёння", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ўчора", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("млрд", 9),
                    ("млн", 6),
                    ("тыс", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("няма", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("альбом", AlbumType::Album),
                    ("шоу", AlbumType::Show),
                    ("сінгл", AlbumType::Single),
                    ("аўдыякніга", AlbumType::Audiobook),
                    ("міні-альбом", AlbumType::Ep),
                ],
            },
        },
        Language::Bg => Entry {
            timeago_tokens: ::phf::Map {
                key: 351906021642186605,
                disps: &[
                    (0, 11),
                    (6, 0),
                    (5, 0),
                    (11, 14),
                ],
                entries: &[
                    ("м", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("час", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("часа", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("минути", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("година", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("сек", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("седм", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("месеца", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("мин", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("седмица", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ден", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("месец", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ч", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("дни", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("секунда", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("минута", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("години", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("г", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("секунди", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("седмици", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("вчера", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("днес", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("млрд", 9),
                    ("хил", 3),
                    ("млн", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("няма", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("сингъл", AlbumType::Single),
                    ("албум", AlbumType::Album),
                    ("аудиокнига", AlbumType::Audiobook),
                    ("миниалбум", AlbumType::Ep),
                    ("предаване", AlbumType::Show),
                ],
            },
        },
        Language::Bn => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 1),
                    (2, 0),
                ],
                entries: &[
                    ("মিনিট", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("সপ\u{9cd}ত\u{9be}হ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("দিন", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ঘন\u{9cd}ট\u{9be}", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ম\u{9be}স", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("বছর", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("সেকেন\u{9cd}ড", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (3, 4),
                    (2, 0),
                    (2, 3),
                ],
                entries: &[
                    ("অক\u{9cd}টো,", 10),
                    ("ফেব,", 2),
                    ("নভে,", 11),
                    ("জ\u{9be}ন\u{9c1},", 1),
                    ("ম\u{9be}র\u{9cd}চ,", 3),
                    ("আগ,", 8),
                    ("ডিসে,", 12),
                    ("জ\u{9c1}ন,", 6),
                    ("এপ\u{9cd}রি,", 4),
                    ("মে,", 5),
                    ("সেপ,", 9),
                    ("জ\u{9c1}ল,", 7),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("গতক\u{9be}ল", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("আজ", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                    (2, 0),
                ],
                entries: &[
                    ("শত", 2),
                    ("হ\u{9be}", 3),
                    ("হ\u{9be}টি", 3),
                    ("কো", 7),
                    ("কোটি", 7),
                    ("ল\u{9be}", 5),
                    ("ল\u{9be}টি", 5),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("০", 0),
                    ("১", 1),
                ],
            },
            album_types: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("সিঙ\u{9cd}গেল", AlbumType::Single),
                    ("শো", AlbumType::Show),
                    ("ইপি", AlbumType::Ep),
                    ("অডিওব\u{9c1}ক", AlbumType::Audiobook),
                    ("অ\u{9cd}য\u{9be}লব\u{9be}ম", AlbumType::Album),
                ],
            },
        },
        Language::Bs => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (14, 26),
                    (5, 10),
                    (0, 9),
                    (0, 7),
                    (1, 0),
                    (8, 20),
                ],
                entries: &[
                    ("godina", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sedmica", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("dan", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("minute", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minutu", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sekunda", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sed", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("d", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sata", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("mjeseca", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("mj", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minuta", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("godine", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("god", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sekundu", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sekunde", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("godinu", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sedmicu", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sat", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("mjesec", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sati", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dana", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sekundi", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("mjeseci", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sedmice", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 9),
                    (1, 6),
                    (5, 0),
                ],
                entries: &[
                    ("jul", 7),
                    ("okt", 10),
                    ("sep", 9),
                    ("nov", 11),
                    ("maj", 5),
                    ("mar", 3),
                    ("apr", 4),
                    ("feb", 2),
                    ("jun", 6),
                    ("jan", 1),
                    ("aug", 8),
                    ("dec", 12),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("jučer", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("danas", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("mil", 6),
                    ("mlr", 9),
                    ("hilj", 3),
                    ("mlrd", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("nema", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (4, 0),
                ],
                entries: &[
                    ("album", AlbumType::Album),
                    ("audio knjiga", AlbumType::Audiobook),
                    ("serija", AlbumType::Show),
                    ("singl", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::Ca => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                    (2, 0),
                    (2, 0),
                    (0, 6),
                ],
                entries: &[
                    ("anys", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("hora", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("any", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("setm", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("segons", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("segon", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("dies", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("dia", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("mesos", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minuts", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("mes", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("hores", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minut", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("setmanes", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("setmana", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (4, 2),
                    (1, 7),
                    (1, 0),
                ],
                entries: &[
                    ("nov", 11),
                    ("jul", 7),
                    ("juny", 6),
                    ("març", 3),
                    ("febr", 2),
                    ("set", 9),
                    ("des", 12),
                    ("d’ag", 8),
                    ("gen", 1),
                    ("d’abr", 4),
                    ("maig", 5),
                    ("d’oct", 10),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("avui", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ahir", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("km", 9),
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("sense", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 106375038446233661,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("àlbum", AlbumType::Album),
                    ("single", AlbumType::Single),
                    ("programa", AlbumType::Show),
                    ("audiollibre", AlbumType::Audiobook),
                ],
            },
        },
        Language::Cs => Entry {
            timeago_tokens: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (2, 20),
                    (0, 0),
                    (4, 12),
                    (0, 13),
                    (1, 16),
                    (21, 31),
                    (10, 3),
                ],
                entries: &[
                    ("hodiny", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("roku", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("týd", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sekund", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("dny", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("měsícem", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("hodinou", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("měs", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("roky", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("dnem", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("týdne", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("rokem", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sekundy", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minut", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minutou", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("měsíce", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minutami", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("lety", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("týdny", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sekundou", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minuty", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sekunda", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("dne", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("l", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("měsíci", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("hodinami", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("týdnem", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sekundami", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("r", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("minuta", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("včera", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("dnes", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("mld", 9),
                    ("tis", 3),
                    ("mil", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            album_types: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("zobrazit", AlbumType::Show),
                    ("audiokniha", AlbumType::Audiobook),
                    ("singl", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("album", AlbumType::Album),
                ],
            },
        },
        Language::Da => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 2),
                    (1, 0),
                    (10, 4),
                    (7, 10),
                ],
                entries: &[
                    ("mdr", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("uge", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("md", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekund", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minutter", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minut", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("år", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("måneder", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekunder", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("uger", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("dag", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("timer", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("måned", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("time", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dage", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 9),
                    (1, 6),
                    (5, 0),
                ],
                entries: &[
                    ("jul", 7),
                    ("okt", 10),
                    ("sep", 9),
                    ("nov", 11),
                    ("maj", 5),
                    ("mar", 3),
                    ("apr", 4),
                    ("feb", 2),
                    ("jun", 6),
                    ("jan", 1),
                    ("aug", 8),
                    ("dec", 12),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("dag", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("går", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("mio", 6),
                    ("mia", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ingen", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (4, 0),
                ],
                entries: &[
                    ("album", AlbumType::Album),
                    ("lyddrama", AlbumType::Show),
                    ("single", AlbumType::Single),
                    ("lydbog", AlbumType::Audiobook),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::De => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (2, 0),
                    (1, 2),
                    (12, 13),
                    (3, 3),
                ],
                entries: &[
                    ("woche", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("std", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("monat", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("stunden", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("sekunden", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("stunde", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("jahren", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("wochen", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("monaten", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minuten", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("tagen", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sekunde", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minute", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("tag", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("jahr", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("gestern", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("heute", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mrd", 9),
                    ("mio", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("keine", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 2980949210194914378,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("album", AlbumType::Album),
                    ("hörbuch", AlbumType::Audiobook),
                    ("single", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("hörspiel", AlbumType::Show),
                ],
            },
        },
        Language::El => Entry {
            timeago_tokens: ::phf::Map {
                key: 399332969041170284,
                disps: &[
                    (0, 0),
                    (1, 10),
                    (3, 15),
                    (0, 0),
                ],
                entries: &[
                    ("δευτερόλεπτο", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("λεπτό", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("έτη", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("δευτ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("εβδομάδα", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("μήνες", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("έτος", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("εβδομάδες", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ημέρες", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("λεπ", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("λ", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("εβδ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ημέρα", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("δευτερόλεπτα", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ώρες", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ημ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ώ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("μήνα", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ώρα", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("λεπτά", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                    (4, 0),
                    (0, 4),
                ],
                entries: &[
                    ("ιουλ", 7),
                    ("δεκ", 12),
                    ("απρ", 4),
                    ("οκτ", 10),
                    ("σεπ", 9),
                    ("φεβ", 2),
                    ("νοε", 11),
                    ("μαΐ", 5),
                    ("ιουν", 6),
                    ("ιαν", 1),
                    ("μαρ", 3),
                    ("αυγ", 8),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("χτες", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("σήμερα", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("εκ", 6),
                    ("χιλ", 3),
                    ("δισ", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("καμία", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ηχητικό βιβλίο", AlbumType::Audiobook),
                    ("single", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("εκπομπή", AlbumType::Show),
                    ("άλμπουμ", AlbumType::Album),
                ],
            },
        },
        Language::En | Language::EnGb | Language::EnIn => Entry {
            timeago_tokens: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (5, 8),
                    (1, 23),
                    (2, 11),
                    (11, 4),
                    (0, 0),
                    (0, 16),
                ],
                entries: &[
                    ("days", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("week", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("day", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("seconds", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minutes", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("month", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("yr", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("d", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("years", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("year", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("hr", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("secs", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("mins", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("months", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("y", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("mo", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minute", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("w", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("weeks", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("wk", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sec", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("m", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("hour", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("hours", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("second", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (8, 0),
                    (5, 8),
                    (4, 0),
                ],
                entries: &[
                    ("nov", 11),
                    ("sept", 9),
                    ("apr", 4),
                    ("dec", 12),
                    ("mar", 3),
                    ("jun", 6),
                    ("sep", 9),
                    ("may", 5),
                    ("jul", 7),
                    ("jan", 1),
                    ("oct", 10),
                    ("feb", 2),
                    ("aug", 8),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("yesterday", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("today", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("crore", 7),
                    ("m", 6),
                    ("lakh", 5),
                    ("b", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("no", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("show", AlbumType::Show),
                    ("ep", AlbumType::Ep),
                    ("single", AlbumType::Single),
                    ("album", AlbumType::Album),
                    ("audiobook", AlbumType::Audiobook),
                ],
            },
        },
        Language::Es => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (7, 0),
                    (6, 15),
                    (0, 16),
                    (0, 0),
                ],
                entries: &[
                    ("m", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("horas", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("semana", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("d", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("semanas", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("años", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("minutos", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("segundo", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("día", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("minuto", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("año", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("segundos", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("mes", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sem", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("días", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("hora", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("meses", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                    (1, 6),
                    (0, 4),
                ],
                entries: &[
                    ("jul", 7),
                    ("oct", 10),
                    ("ene", 1),
                    ("may", 5),
                    ("sept", 9),
                    ("feb", 2),
                    ("nov", 11),
                    ("dic", 12),
                    ("abr", 4),
                    ("jun", 6),
                    ("ago", 8),
                    ("mar", 3),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ayer", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("hoy", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mil", 3),
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            album_types: ::phf::Map {
                key: 2126027241312876569,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("álbum", AlbumType::Album),
                    ("single", AlbumType::Single),
                    ("audiolibro", AlbumType::Audiobook),
                    ("audiodrama", AlbumType::Show),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::EsUs | Language::Es419 => Entry {
            timeago_tokens: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (9, 0),
                    (1, 3),
                    (2, 7),
                    (7, 6),
                ],
                entries: &[
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minuto", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("día", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minutos", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("semanas", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("hora", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("segundo", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("meses", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("año", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("días", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("m", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("mes", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("años", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("semana", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("horas", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("segundos", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sem", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                    (1, 6),
                    (0, 4),
                ],
                entries: &[
                    ("jul", 7),
                    ("oct", 10),
                    ("ene", 1),
                    ("may", 5),
                    ("sept", 9),
                    ("feb", 2),
                    ("nov", 11),
                    ("dic", 12),
                    ("abr", 4),
                    ("jun", 6),
                    ("ago", 8),
                    ("mar", 3),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ayer", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("hoy", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mil", 3),
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("sin", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("audiolibro", AlbumType::Audiobook),
                    ("sencillo", AlbumType::Single),
                    ("programa", AlbumType::Show),
                    ("álbum", AlbumType::Album),
                ],
            },
        },
        Language::Et => Entry {
            timeago_tokens: ::phf::Map {
                key: 12213676231523076107,
                disps: &[
                    (3, 0),
                    (1, 0),
                    (3, 9),
                    (6, 17),
                ],
                entries: &[
                    ("minut", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("nädala", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("nädal", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("p", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("t", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("kuu", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minutit", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("tunni", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("näd", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("aasta", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sekundi", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minuti", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("kuud", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekund", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("päev", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("a", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sekundit", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("päeva", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (4, 0),
                    (2, 4),
                    (0, 3),
                ],
                entries: &[
                    ("okt", 10),
                    ("aug", 8),
                    ("dets", 12),
                    ("mai", 5),
                    ("jaan", 1),
                    ("nov", 11),
                    ("juuni", 6),
                    ("märts", 3),
                    ("sept", 9),
                    ("apr", 4),
                    ("veebr", 2),
                    ("juuli", 7),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("eile", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("täna", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("mld", 9),
                    ("mln", 6),
                    ("tuh", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("pole", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("sari", AlbumType::Show),
                    ("singel", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("audioraamat", AlbumType::Audiobook),
                    ("album", AlbumType::Album),
                ],
            },
        },
        Language::Eu => Entry {
            timeago_tokens: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (5, 9),
                    (1, 0),
                ],
                entries: &[
                    ("ordu", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("hilabete", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("egun", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("urte", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("segundo", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("aste", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("minutu", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::D],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                    (9, 5),
                    (2, 4),
                ],
                entries: &[
                    ("uzt", 7),
                    ("aza", 11),
                    ("ots", 2),
                    ("urr", 10),
                    ("mai", 5),
                    ("urt", 1),
                    ("abu", 8),
                    ("eka", 6),
                    ("abe", 12),
                    ("ira", 9),
                    ("mar", 3),
                    ("api", 4),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("atzo", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("gaur", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ez", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 2980949210194914378,
                disps: &[
                    (4, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("singlea", AlbumType::Single),
                    ("saioa", AlbumType::Show),
                    ("audio-liburua", AlbumType::Audiobook),
                    ("albuma", AlbumType::Album),
                ],
            },
        },
        Language::Fa => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                    (0, 1),
                ],
                entries: &[
                    ("ماه", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("روز", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("هفته", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ساعت", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ثانیه", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("سال", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("دقیقه", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                    (4, 3),
                    (7, 8),
                ],
                entries: &[
                    ("دسامبر", 12),
                    ("ژانویه", 1),
                    ("مارس", 3),
                    ("ژوئیه", 7),
                    ("سپتامبر", 9),
                    ("آوریل", 4),
                    ("نوامبر", 11),
                    ("ژوئن", 6),
                    ("مه", 5),
                    ("اوت", 8),
                    ("اکتبر", 10),
                    ("فوریه", 2),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("امروز", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("دیروز", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("میلیارد", 9),
                    ("میلیون", 6),
                    ("هزار", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("بدون", 0),
                    ("۱", 1),
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("آلبوم", AlbumType::Album),
                    ("تک آهنگ", AlbumType::Single),
                    ("نمایش", AlbumType::Show),
                    ("پخش فوق\u{200c}العاده", AlbumType::Ep),
                    ("کتاب صوتی", AlbumType::Audiobook),
                ],
            },
        },
        Language::Fi => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (1, 6),
                    (2, 4),
                    (0, 0),
                    (4, 7),
                    (0, 1),
                ],
                entries: &[
                    ("tunti", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("vuotta", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("kk", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekunti", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("vk", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("vuosi", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("t", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("sekuntia", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("kuukautta", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("v", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("tuntia", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("pv", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("viikkoa", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("päivä", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("kuukausi", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minuutti", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minuuttia", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("päivää", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("viikko", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("tänään", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("eilen", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mrd", 9),
                    ("t", 3),
                    ("milj", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ei", 0),
                    ("katselukertoja", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 8602556344903797927,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("äänikirja", AlbumType::Audiobook),
                    ("single", AlbumType::Single),
                    ("albumi", AlbumType::Album),
                    ("näytä", AlbumType::Show),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::Fil => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                    (5, 2),
                ],
                entries: &[
                    ("segundo", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("araw", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("minuto", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("oras", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("taon", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("seg", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("buwan", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("linggo", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                    (7, 3),
                    (4, 3),
                ],
                entries: &[
                    ("abr", 4),
                    ("set", 9),
                    ("mar", 3),
                    ("may", 5),
                    ("hun", 6),
                    ("nob", 11),
                    ("okt", 10),
                    ("ene", 1),
                    ("hul", 7),
                    ("dis", 12),
                    ("ago", 8),
                    ("peb", 2),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ngayong", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("kahapon", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("b", 9),
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("walang", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("audiobook", AlbumType::Audiobook),
                    ("single", AlbumType::Single),
                    ("palabas", AlbumType::Show),
                    ("album", AlbumType::Album),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::Fr | Language::FrCa => Entry {
            timeago_tokens: ::phf::Map {
                key: 2980949210194914378,
                disps: &[
                    (3, 1),
                    (7, 0),
                    (17, 1),
                    (1, 11),
                ],
                entries: &[
                    ("m", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minutes", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("semaines", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("j", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("an", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("heures", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("semaine", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ans", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("jours", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("seconde", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("heure", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minute", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("secondes", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("mois", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sem", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("jour", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (10, 7),
                    (1, 0),
                    (3, 1),
                ],
                entries: &[
                    ("août", 8),
                    ("févr", 2),
                    ("nov", 11),
                    ("sept", 9),
                    ("janv", 1),
                    ("oct", 10),
                    ("juin", 6),
                    ("déc", 12),
                    ("juil", 7),
                    ("mars", 3),
                    ("avr", 4),
                    ("mai", 5),
                    ("juill", 7),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("hier", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("aujourd'hui", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("g", 9),
                    ("md", 9),
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("aucune", 0),
                    ("aucun", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                    (5, 0),
                ],
                entries: &[
                    ("microalbum", AlbumType::Ep),
                    ("album", AlbumType::Album),
                    ("livre audio", AlbumType::Audiobook),
                    ("émission", AlbumType::Show),
                    ("simple", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("single", AlbumType::Single),
                ],
            },
        },
        Language::Gl => Entry {
            timeago_tokens: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (2, 4),
                    (12, 1),
                    (14, 2),
                    (0, 0),
                ],
                entries: &[
                    ("día", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("minuto", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("semanas", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("días", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("segundos", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("meses", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("mes", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("horas", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("sem", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("anos", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("segundo", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ano", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("hora", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minutos", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("semana", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (2, 0),
                    (2, 10),
                    (2, 5),
                ],
                entries: &[
                    ("nov", 11),
                    ("abr", 4),
                    ("maio", 5),
                    ("ago", 8),
                    ("mar", 3),
                    ("xuño", 6),
                    ("xul", 7),
                    ("out", 10),
                    ("set", 9),
                    ("feb", 2),
                    ("xan", 1),
                    ("dec", 12),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("hoxe", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("onte", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ningunha", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("álbum", AlbumType::Album),
                    ("single", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("programa", AlbumType::Show),
                    ("audiolibro", AlbumType::Audiobook),
                ],
            },
        },
        Language::Gu => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 1),
                    (4, 0),
                ],
                entries: &[
                    ("દિવસ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("અઠવાડિયા", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("મિનિટ", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("મહિના", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("અઠ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("સ\u{ac7}ક\u{a82}ડ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("વર\u{acd}ષ", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("કલાક", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                    (3, 11),
                    (4, 6),
                ],
                entries: &[
                    ("ઑગસ\u{acd}ટ,", 8),
                    ("ફ\u{ac7}બ\u{acd}ર\u{ac1},", 2),
                    ("મ\u{ac7},", 5),
                    ("માર\u{acd}ચ,", 3),
                    ("નવ\u{ac7},", 11),
                    ("જાન\u{acd}ય\u{ac1},", 1),
                    ("એપ\u{acd}રિલ,", 4),
                    ("જ\u{ac1}લાઈ,", 7),
                    ("સપ\u{acd}ટ\u{ac7},", 9),
                    ("જ\u{ac2}ન,", 6),
                    ("ડિસ\u{ac7},", 12),
                    ("ઑક\u{acd}ટો,", 10),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("આજ\u{ac7}", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ગઈકાલ\u{ac7}", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("કરોડ", 7),
                    ("અબજ", 9),
                    ("હજાર", 3),
                    ("લાખ", 5),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            album_types: ::phf::Map {
                key: 2980949210194914378,
                disps: &[
                    (4, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("આલ\u{acd}બમ", AlbumType::Album),
                    ("શો", AlbumType::Show),
                    ("ઑડિયોબ\u{ac1}ક", AlbumType::Audiobook),
                    ("સિ\u{a82}ગલ", AlbumType::Single),
                ],
            },
        },
        Language::Hi => Entry {
            timeago_tokens: ::phf::Map {
                key: 12213676231523076107,
                disps: &[
                    (2, 0),
                    (1, 8),
                ],
                entries: &[
                    ("घ\u{902}ट\u{947}", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("मिनट", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("सप\u{94d}ताह", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("वर\u{94d}ष", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("माह", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("घ\u{902}॰", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("दिन", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("मि॰", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("स\u{947}॰", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("स\u{947}क\u{902}ड", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (7, 0),
                    (2, 8),
                    (0, 2),
                ],
                entries: &[
                    ("मार\u{94d}च", 3),
                    ("सित॰", 9),
                    ("फ\u{93c}र॰", 2),
                    ("नव॰", 11),
                    ("ज\u{942}न", 6),
                    ("अग॰", 8),
                    ("ज\u{941}ल॰", 7),
                    ("जन॰", 1),
                    ("मई", 5),
                    ("अक\u{94d}त\u{942}॰", 10),
                    ("दिस॰", 12),
                    ("अप\u{94d}र\u{948}ल", 4),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("कल", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("आज", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 2980949210194914378,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("हज\u{93c}ार", 3),
                    ("लाख", 5),
                    ("क॰", 7),
                    ("अ॰", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("नही\u{902}", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("ऑडियो ब\u{941}क", AlbumType::Audiobook),
                    ("शो", AlbumType::Show),
                    ("सि\u{902}गल", AlbumType::Single),
                    ("ईपी", AlbumType::Ep),
                    ("एल\u{94d}\u{200d}बम", AlbumType::Album),
                ],
            },
        },
        Language::Hr => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 16),
                    (7, 4),
                    (0, 6),
                    (0, 1),
                    (2, 0),
                    (0, 8),
                ],
                entries: &[
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minutu", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minuta", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("mjeseci", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekunde", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sata", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("sekundi", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("mj", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("tjedna", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("godine", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("godinu", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("dana", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sat", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("mjesec", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sati", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dan", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("tjedana", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("tjedan", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sekunda", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("tj", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("minute", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("mjeseca", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("godina", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sekundu", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("g", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 8),
                    (0, 4),
                    (2, 0),
                ],
                entries: &[
                    ("lis", 10),
                    ("stu", 11),
                    ("velj", 2),
                    ("lip", 6),
                    ("svi", 5),
                    ("ruj", 9),
                    ("pro", 12),
                    ("sij", 1),
                    ("kol", 8),
                    ("ožu", 3),
                    ("tra", 4),
                    ("srp", 7),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("jučer", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("danas", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mlr", 9),
                    ("tis", 3),
                    ("mil", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("nema", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("serija", AlbumType::Show),
                    ("audioknjiga", AlbumType::Audiobook),
                    ("album", AlbumType::Album),
                    ("singl", AlbumType::Single),
                ],
            },
        },
        Language::Hu => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (6, 1),
                    (3, 0),
                    (1, 5),
                    (0, 12),
                ],
                entries: &[
                    ("hónappal", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("perce", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ó", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("évvel", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("másodperccel", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("perc", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("órája", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("héttel", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("hónapja", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("nappal", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("hete", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("perccel", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("p", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("mp", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("másodperc", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("napja", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("másodperce", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("éve", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("órával", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::D],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (5, 0),
                    (0, 10),
                    (6, 9),
                ],
                entries: &[
                    ("jan", 1),
                    ("jún", 6),
                    ("júl", 7),
                    ("márc", 3),
                    ("máj", 5),
                    ("dec", 12),
                    ("okt", 10),
                    ("ápr", 4),
                    ("nov", 11),
                    ("aug", 8),
                    ("szept", 9),
                    ("febr", 2),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("tegnap", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ma", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("e", 3),
                    ("mrd", 9),
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("nincs", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 2980949210194914378,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("album", AlbumType::Album),
                    ("kislemez", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("hangoskönyv", AlbumType::Audiobook),
                    ("műsor", AlbumType::Show),
                ],
            },
        },
        Language::Hy => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (4, 2),
                    (3, 6),
                    (2, 0),
                ],
                entries: &[
                    ("տարի", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ժ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ժամ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("տ", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("վրկ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("շաբաթ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("վայրկյան", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("շաբ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("օր", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ր", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("րոպե", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ամիս", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (3, 0),
                    (1, 0),
                    (3, 11),
                ],
                entries: &[
                    ("հոկ,", 10),
                    ("սեպ,", 9),
                    ("հլս,", 7),
                    ("նոյ,", 11),
                    ("մրտ,", 3),
                    ("փտվ,", 2),
                    ("դեկ,", 12),
                    ("օգս,", 8),
                    ("հնս,", 6),
                    ("հնվ,", 1),
                    ("մյս,", 5),
                    ("ապր,", 4),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("երեկ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("այսօր", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("հզր", 3),
                    ("մլն", 6),
                    ("մլրդ", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("դիտումներ", 0),
                    ("չկան", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("աուդիոդրամա", AlbumType::Show),
                    ("աուդիոգիրք", AlbumType::Audiobook),
                    ("սինգլ", AlbumType::Single),
                    ("ալբոմ", AlbumType::Album),
                ],
            },
        },
        Language::Id => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (1, 0),
                    (5, 3),
                    (4, 5),
                ],
                entries: &[
                    ("mgg", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("bln", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("detik", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("minggu", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("mnt", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("bulan", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("hari", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("jam", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("thn", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("dtk", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("tahun", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("menit", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 106375038446233661,
                disps: &[
                    (0, 0),
                    (1, 7),
                    (0, 1),
                ],
                entries: &[
                    ("feb", 2),
                    ("mar", 3),
                    ("jul", 7),
                    ("jan", 1),
                    ("agu", 8),
                    ("nov", 11),
                    ("mei", 5),
                    ("jun", 6),
                    ("des", 12),
                    ("sep", 9),
                    ("okt", 10),
                    ("apr", 4),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ini", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("kemarin", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("rb", 3),
                    ("jt", 6),
                    ("m", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("belum", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (4, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("single", AlbumType::Single),
                    ("buku audio", AlbumType::Audiobook),
                    ("album", AlbumType::Album),
                    ("acara", AlbumType::Show),
                ],
            },
        },
        Language::Is => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 20),
                    (0, 0),
                    (0, 4),
                    (3, 3),
                    (3, 3),
                ],
                entries: &[
                    ("vikum", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("klukkustund", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ári", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("árum", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sekúndur", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("mínútur", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("dögum", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("klst", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("mánuði", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("viku", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("mínúta", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("mínútu", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("mínútum", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("mín", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("mánuðum", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekúnda", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sekúndum", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("klukkustundum", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("sekúndu", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("degi", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("mán", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (6, 0),
                    (0, 9),
                    (3, 7),
                ],
                entries: &[
                    ("apr", 4),
                    ("júl", 7),
                    ("feb", 2),
                    ("nóv", 11),
                    ("ágú", 8),
                    ("okt", 10),
                    ("des", 12),
                    ("sep", 9),
                    ("mar", 3),
                    ("maí", 5),
                    ("jan", 1),
                    ("jún", 6),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("dag", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("gær", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("þ", 3),
                    ("ma", 9),
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("eitt", 1),
                    ("ekkert", 0),
                    ("einn", 1),
                ],
            },
            album_types: ::phf::Map {
                key: 12213676231523076107,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("hljóðbók", AlbumType::Audiobook),
                    ("þáttur", AlbumType::Show),
                    ("smáskífa", AlbumType::Single),
                    ("plata", AlbumType::Album),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::It => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (2, 3),
                    (1, 0),
                    (7, 2),
                    (0, 3),
                    (7, 12),
                ],
                entries: &[
                    ("minuto", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("anno", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("giorni", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ora", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("sec", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("settimane", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("gg", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sett", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("mese", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("settimana", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("secondi", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("g", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ore", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("anni", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("giorno", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("mesi", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minuti", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("secondo", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (0, 0),
                    (0, 2),
                    (3, 1),
                ],
                entries: &[
                    ("apr", 4),
                    ("set", 9),
                    ("mag", 5),
                    ("gen", 1),
                    ("lug", 7),
                    ("nov", 11),
                    ("feb", 2),
                    ("ott", 10),
                    ("mar", 3),
                    ("dic", 12),
                    ("ago", 8),
                    ("giu", 6),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ieri", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("oggi", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mio", 6),
                    ("mln", 6),
                    ("mrd", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("nessuna", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("audiolibro", AlbumType::Audiobook),
                    ("album", AlbumType::Album),
                    ("singolo", AlbumType::Single),
                    ("programma", AlbumType::Show),
                ],
            },
        },
        Language::Iw => Entry {
            timeago_tokens: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (0, 0),
                    (0, 8),
                    (0, 11),
                    (1, 3),
                    (1, 22),
                    (17, 18),
                ],
                entries: &[
                    ("יום", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("שנים", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("שנה", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("שעתיים", TaToken { n: 2, unit: Some(TimeUnit::Hour) }),
                    ("שנ׳", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("יומיים", TaToken { n: 2, unit: Some(TimeUnit::Day) }),
                    ("שבוע", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("שתי", TaToken { n: 2, unit: None }),
                    ("חודשיים", TaToken { n: 2, unit: Some(TimeUnit::Month) }),
                    ("שניה", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("שניות", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("שבועות", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("דקה", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("שנתיים", TaToken { n: 2, unit: Some(TimeUnit::Year) }),
                    ("שעה", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("שעות", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("חודשים", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("דק׳", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("אתמול", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("שב׳", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("חודש", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("שבועיים", TaToken { n: 2, unit: Some(TimeUnit::Week) }),
                    ("ימים", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("שנייה", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("שע׳", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("דקות", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                    (6, 1),
                    (0, 3),
                ],
                entries: &[
                    ("ביולי", 7),
                    ("ביוני", 6),
                    ("במרץ", 3),
                    ("בספט׳", 9),
                    ("בינו׳", 1),
                    ("באוג׳", 8),
                    ("במאי", 5),
                    ("בדצמ׳", 12),
                    ("באוק׳", 10),
                    ("בנוב׳", 11),
                    ("באפר׳", 4),
                    ("בפבר׳", 2),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("היום", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("אתמול", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("b", 9),
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("אין", 0),
                    ("אחד", 1),
                ],
            },
            album_types: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("אלבום", AlbumType::Album),
                    ("תסכית", AlbumType::Show),
                    ("מיני-אלבום", AlbumType::Ep),
                    ("ספר אודיו", AlbumType::Audiobook),
                    ("סינגל", AlbumType::Single),
                ],
            },
        },
        Language::Ja => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                    (5, 0),
                ],
                entries: &[
                    ("時", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("週", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("月", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("日", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("秒", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("年", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("分", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::M, DateCmp::D],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("本", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("昨", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("万", 4),
                    ("億", 8),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("アルバム", AlbumType::Album),
                    ("シングル", AlbumType::Single),
                    ("オーディオブック", AlbumType::Audiobook),
                    ("表示", AlbumType::Show),
                ],
            },
        },
        Language::Ka => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (10, 4),
                    (1, 0),
                    (5, 10),
                ],
                entries: &[
                    ("წამი", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("თვის", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("კვირის", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("წუთის", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("წამის", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("წუთი", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("წლის", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("საათის", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("წთ", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("სთ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("დღის", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("კვ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("წმ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (0, 5),
                    (1, 0),
                    (6, 5),
                ],
                entries: &[
                    ("თებ", 2),
                    ("აგვ", 8),
                    ("ნოე", 11),
                    ("აპრ", 4),
                    ("დეკ", 12),
                    ("სექ", 9),
                    ("ოქტ", 10),
                    ("ივნ", 6),
                    ("მაი", 5),
                    ("ივლ", 7),
                    ("იან", 1),
                    ("მარ", 3),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("გუშინ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("დღეს", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("მლნ", 6),
                    ("მლრდ", 9),
                    ("ათ", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("არ", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("მინი-ალბომი", AlbumType::Ep),
                    ("აუდიოწიგნი", AlbumType::Audiobook),
                    ("ალბომი", AlbumType::Album),
                    ("ჩვენება", AlbumType::Show),
                    ("სინგლი", AlbumType::Single),
                ],
            },
        },
        Language::Kk => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (2, 0),
                    (4, 6),
                    (0, 7),
                ],
                entries: &[
                    ("сағат", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("жыл", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("апта", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ж", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("сағ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("күн", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ай", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("сек", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("минут", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ап", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("секунд", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("с", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("мин", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::D],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (3, 1),
                    (1, 0),
                    (0, 6),
                ],
                entries: &[
                    ("там", 8),
                    ("шіл", 7),
                    ("мау", 6),
                    ("мам", 5),
                    ("нау", 3),
                    ("қаз", 10),
                    ("жел", 12),
                    ("ақп", 2),
                    ("қар", 11),
                    ("сәу", 4),
                    ("қыр", 9),
                    ("қаң", 1),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("кеше", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("бүгін", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("мың", 3),
                    ("млрд", 9),
                    ("м", 3),
                    ("млн", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ешкім", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 351906021642186605,
                disps: &[
                    (4, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("аудиокітап", AlbumType::Audiobook),
                    ("шоу", AlbumType::Show),
                    ("сингл", AlbumType::Single),
                    ("альбом", AlbumType::Album),
                ],
            },
        },
        Language::Km => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (7, 3),
                    (7, 0),
                ],
                entries: &[
                    ("វ\u{17b7}នាទ\u{17b8}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ខែម\u{17bb}ន", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ថ\u{17d2}ងៃម\u{17bb}ន", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ម\u{17c9}ោងម\u{17bb}ន", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ឆ\u{17d2}នា\u{17c6}ម\u{17bb}ន", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("សប\u{17d2}ដាហ\u{17cd}ម\u{17bb}ន", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("វ\u{17b7}នាទ\u{17b8}ម\u{17bb}ន", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("នាទ\u{17b8}ម\u{17bb}ន", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("នាទ\u{17b8}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (10, 0),
                    (3, 0),
                    (0, 10),
                ],
                entries: &[
                    ("វ\u{17b7}ច\u{17d2}ឆ\u{17b7}កា", 11),
                    ("ម\u{17b7}ថ\u{17bb}នា", 6),
                    ("ម\u{17b8}នា", 3),
                    ("ឧសភា", 5),
                    ("មករា", 1),
                    ("ក\u{17bb}ម\u{17d2}ភៈ", 2),
                    ("មេសា", 4),
                    ("កញ\u{17d2}ញា", 9),
                    ("ត\u{17bb}លា", 10),
                    ("ស\u{17b8}ហា", 8),
                    ("កក\u{17d2}កដា", 7),
                    ("ធ\u{17d2}ន\u{17bc}", 12),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("បានធ\u{17d2}វើបច\u{17d2}ច\u{17bb}ប\u{17d2}បន\u{17d2}នភាពនៅថ\u{17d2}ងៃនេះ", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("បានធ\u{17d2}វើបច\u{17d2}ច\u{17bb}ប\u{17d2}បន\u{17d2}នភាពម\u{17d2}ស\u{17b7}លម\u{17b7}ញ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("លាន", 6),
                    ("ប\u{17ca}\u{17b8}លាន", 9),
                    ("ពាន\u{17cb}", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            album_types: ::phf::Map {
                key: 12213676231523076107,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("អាល\u{17cb}ប\u{17ca}\u{17bb}ម", AlbumType::Album),
                    ("សៀវភៅ\u{200b}ជា\u{200b}ស\u{17c6}ឡេង", AlbumType::Audiobook),
                    ("កម\u{17d2}មវ\u{17b7}ធ\u{17b8}ទ\u{17bc}រទស\u{17d2}សន\u{17cd}", AlbumType::Show),
                    ("ep", AlbumType::Ep),
                    ("ចម\u{17d2}រៀងទោល", AlbumType::Single),
                ],
            },
        },
        Language::Kn => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 1),
                    (0, 6),
                    (4, 7),
                    (0, 0),
                    (0, 17),
                ],
                entries: &[
                    ("ದ\u{cbf}ನದ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ವರ\u{ccd}ಷಗಳ", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ನ\u{cbf}ಮ\u{cbf}ಷಗಳು", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ಗಂಟ\u{cc6}ಗಳ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ತ\u{cbf}ಂಗಳು", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ನ\u{cbf}ಮ\u{cbf}ಷಗಳ", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ವಾರಗಳ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ತ\u{cbf}ಂಗಳುಗಳ", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ಗಂ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ನ\u{cbf}ಮ\u{cbf}ಷದ", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ಸ\u{cc6}ಕ\u{cc6}ಂಡುಗಳು", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ನ\u{cbf}ಮ\u{cbf}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ಗಂಟ\u{cc6}", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ವಾರದ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ಸ\u{cc6}ಕ\u{cc6}ಂಡುಗಳ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ತ\u{cbf}ಂಗಳ", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ಸ\u{cc6}ಕ\u{cc6}ಂ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ನ\u{cbf}ಮ\u{cbf}ಷವು", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ವರ\u{ccd}ಷದ", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ದ\u{cbf}ನಗಳ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ಸ\u{cc6}ಕ\u{cc6}ಂಡ\u{ccd}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (3, 0),
                    (5, 2),
                    (2, 6),
                ],
                entries: &[
                    ("ಮಾರ\u{ccd}ಚ\u{ccd}", 3),
                    ("ಡ\u{cbf}ಸ\u{cc6}ಂ", 12),
                    ("ನವ\u{cc6}ಂ", 11),
                    ("ಫ\u{cc6}ಬ\u{ccd}ರವರ\u{cbf}", 2),
                    ("ಜ\u{cc2}ನ\u{ccd}", 6),
                    ("ಮೇ", 5),
                    ("ಜನವರ\u{cbf}", 1),
                    ("ಅಕ\u{ccd}ಟೋ", 10),
                    ("ಏಪ\u{ccd}ರ\u{cbf}", 4),
                    ("ಆಗಸ\u{ccd}ಟ\u{ccd}", 8),
                    ("ಜುಲೈ", 7),
                    ("ಸ\u{cc6}ಪ\u{ccd}ಟ\u{cc6}ಂ", 9),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ಇಂದು", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ನ\u{cbf}ನ\u{ccd}ನ\u{cc6}", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ಕೋಟ\u{cbf}", 7),
                    ("ಲಕ\u{ccd}ಷ", 5),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ವೀಕ\u{ccd}ಷಣ\u{cc6}ಗಳ\u{cbf}ಲ\u{ccd}ಲ", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 9826310008611304355,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("ಆಡ\u{cbf}ಯೋಬುಕ\u{ccd}", AlbumType::Audiobook),
                    ("ಶೋ", AlbumType::Show),
                    ("ಒಂದೇ", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("ಆಲ\u{ccd}ಬಮ\u{ccd}", AlbumType::Album),
                ],
            },
        },
        Language::Ko => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (4, 1),
                    (5, 0),
                ],
                entries: &[
                    ("분", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("일", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("시간", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("개월", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("년", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("초", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("주", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::M, DateCmp::D],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("어제", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("오늘", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("천", 3),
                    ("만", 4),
                    ("억", 8),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("없", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 12213676231523076107,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("표시", AlbumType::Show),
                    ("오디오북", AlbumType::Audiobook),
                    ("앨범", AlbumType::Album),
                    ("ep", AlbumType::Ep),
                    ("싱글", AlbumType::Single),
                ],
            },
        },
        Language::Ky => Entry {
            timeago_tokens: ::phf::Map {
                key: 106375038446233661,
                disps: &[
                    (0, 5),
                    (7, 0),
                ],
                entries: &[
                    ("апт", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("мүнөт", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("күн", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("апта", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("саат", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("сек", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("мүн", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("секунд", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("жыл", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ай", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::D],
            months: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (0, 3),
                    (6, 10),
                    (2, 0),
                ],
                entries: &[
                    ("май", 5),
                    ("фев", 2),
                    ("янв", 1),
                    ("ноя", 11),
                    ("июн", 6),
                    ("авг", 8),
                    ("сен", 9),
                    ("дек", 12),
                    ("мар", 3),
                    ("окт", 10),
                    ("апр", 4),
                    ("июл", 7),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("кечээ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("бүгүн", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("млд", 9),
                    ("млн", 6),
                    ("миң", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("эч", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("альбом", AlbumType::Album),
                    ("шоу", AlbumType::Show),
                    ("аудиокитеп", AlbumType::Audiobook),
                    ("сингл", AlbumType::Single),
                    ("чакан альбом", AlbumType::Ep),
                ],
            },
        },
        Language::Lo => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                    (11, 6),
                    (1, 0),
                ],
                entries: &[
                    ("ດ", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ວ\u{eb4}ນາທ\u{eb5}ກ\u{ec8}ອນ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ມ\u{eb7}\u{ec9}ກ\u{ec8}ອນ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ປ\u{eb5}ກ\u{ec8}ອນ", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ຊ\u{ebb}\u{ec8}ວໂມງກ\u{ec8}ອນ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ເດ\u{eb7}ອນກ\u{ec8}ອນ", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ວ\u{eb4}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ນາທ\u{eb5}ກ\u{ec8}ອນ", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ນທ", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ອາທ\u{eb4}ດກ\u{ec8}ອນ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ອທ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ນາທ\u{eb5}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ວ\u{eb4}ນາທ\u{eb5}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ຊມ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 106375038446233661,
                disps: &[
                    (1, 0),
                    (5, 9),
                    (2, 0),
                ],
                entries: &[
                    ("ມນ", 3),
                    ("ກຍ", 9),
                    ("ມ\u{eb4}ຖ", 6),
                    ("ພພ", 5),
                    ("ກລ", 7),
                    ("ມກ", 1),
                    ("ມສ", 4),
                    ("ສຫ", 8),
                    ("ກພ", 2),
                    ("ພຈ", 11),
                    ("ທວ", 12),
                    ("ຕລ", 10),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ອ\u{eb1}ບເດດມ\u{eb7}\u{ec9}ນ\u{eb5}\u{ec9}", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ອ\u{eb1}ດເດດມ\u{eb7}\u{ec9}ວານນ\u{eb5}\u{ec9}", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("ກ\u{eb5}ບ", 3),
                    ("ລ\u{ec9}ານ", 6),
                    ("ຕ\u{eb7}\u{ec9}", 9),
                    ("ພ\u{eb1}ນ", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ຍ\u{eb1}ງບ\u{ecd}\u{ec8}ມ\u{eb5}ຄ\u{ebb}ນເບ\u{eb4}\u{ec8}ງເທ\u{eb7}\u{ec8}ອ", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("ຊ\u{eb4}ງເກ\u{eb4}ນ", AlbumType::Single),
                    ("ອະລະບ\u{ecd}\u{ec9}າ", AlbumType::Album),
                    ("ປ\u{eb6}\u{ec9}ມສຽງ", AlbumType::Audiobook),
                    ("ສະແດງ", AlbumType::Show),
                ],
            },
        },
        Language::Lt => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 12),
                    (1, 15),
                    (0, 4),
                    (0, 33),
                    (3, 0),
                    (0, 13),
                    (5, 33),
                ],
                entries: &[
                    ("valandos", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("sekundė", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("valandą", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minutė", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("val", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("mėnesius", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("d", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sekundžių", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minutę", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("dienas", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("valandų", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("savaitę", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("valandas", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("metus", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("savaičių", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("mėnesį", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sav", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("metų", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sekundes", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("dienos", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("mėnesių", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("savaites", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("minučių", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sekundę", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minutės", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minutes", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("savaitės", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sekundės", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("mėn", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("dienų", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("m", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("mėnesio", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("dieną", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::M, DateCmp::D],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("šiandien", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("vakar", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mlrd", 9),
                    ("mln", 6),
                    ("tūkst", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("nėra", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("mini albumas", AlbumType::Ep),
                    ("serialas", AlbumType::Show),
                    ("singlas", AlbumType::Single),
                    ("albumas", AlbumType::Album),
                    ("garsinė knyga", AlbumType::Audiobook),
                ],
            },
        },
        Language::Lv => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (14, 1),
                    (12, 11),
                    (5, 0),
                    (4, 0),
                    (2, 6),
                ],
                entries: &[
                    ("nedēļas", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("mēn", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("nedēļām", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("gada", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("minūšu", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minūte", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minūtes", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("d", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("minūtēm", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("dienas", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("gadiem", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("stundas", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("sekunžu", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("dienām", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("mēneša", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("g", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sekundēm", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("stundām", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("mēnešiem", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekunde", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("st", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("sekundes", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ned", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::D],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (3, 11),
                    (0, 6),
                    (0, 0),
                ],
                entries: &[
                    ("sept", 9),
                    ("marts", 3),
                    ("febr", 2),
                    ("jūn", 6),
                    ("dec", 12),
                    ("nov", 11),
                    ("apr", 4),
                    ("jūl", 7),
                    ("maijs", 5),
                    ("janv", 1),
                    ("aug", 8),
                    ("okt", 10),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("šodien", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("vakar", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("milj", 6),
                    ("mljrd", 9),
                    ("tūkst", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("nav", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("singls", AlbumType::Single),
                    ("audiogrāmata", AlbumType::Audiobook),
                    ("albums", AlbumType::Album),
                    ("ep ieraksts", AlbumType::Ep),
                    ("pārraide", AlbumType::Show),
                ],
            },
        },
        Language::Mk => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 7),
                    (0, 5),
                    (0, 0),
                    (5, 3),
                ],
                entries: &[
                    ("година", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("минута", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("дена", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("минути", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("сек", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("месеци", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("седмица", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("час", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("секунда", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("часа", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("год", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ден", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("години", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("мин", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("седмици", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("месец", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("секунди", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("вчера", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("денес", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("мил", 6),
                    ("м", 6),
                    ("милј", 9),
                    ("илј", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("нема", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("аудиокнига", AlbumType::Audiobook),
                    ("ep", AlbumType::Ep),
                    ("албум", AlbumType::Album),
                    ("сингл", AlbumType::Single),
                    ("серија", AlbumType::Show),
                ],
            },
        },
        Language::Ml => Entry {
            timeago_tokens: ::phf::Map {
                key: 2980949210194914378,
                disps: &[
                    (9, 5),
                    (6, 0),
                ],
                entries: &[
                    ("മ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("മണിക\u{d4d}ക\u{d42}ർ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("സെ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("മ\u{d3e}സം", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ആഴ\u{d4d}ച", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ദിവസം", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("മിനിറ\u{d4d}റ\u{d4d}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("മി", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("വർഷം", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("സെക\u{d4d}കൻഡ\u{d4d}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::D],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (10, 0),
                    (1, 11),
                    (1, 6),
                ],
                entries: &[
                    ("ഫെബ\u{d4d}ര\u{d41}", 2),
                    ("മേയ\u{d4d}", 5),
                    ("ഓഗ", 8),
                    ("ഒക\u{d4d}ടോ", 10),
                    ("ജ\u{d42}ലൈ", 7),
                    ("മ\u{d3e}ർ", 3),
                    ("ഏപ\u{d4d}രി", 4),
                    ("ജന\u{d41}", 1),
                    ("ജ\u{d42}ൺ", 6),
                    ("നവം", 11),
                    ("സെപ\u{d4d}റ\u{d4d}റം", 9),
                    ("ഡിസം", 12),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ഇന\u{d4d}ന\u{d4d}", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ഇന\u{d4d}നലെ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ലക\u{d4d}ഷം", 5),
                    ("കോടി", 7),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ഇല\u{d4d}ല", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("സിംഗിൾ", AlbumType::Single),
                    ("ഓഡിയോ ബ\u{d41}ക\u{d4d}ക\u{d4d}", AlbumType::Audiobook),
                    ("ep", AlbumType::Ep),
                    ("ഷോ", AlbumType::Show),
                    ("ആല\u{d4d}\u{200d}\u{200c}ബം", AlbumType::Album),
                ],
            },
        },
        Language::Mn => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 2),
                    (2, 0),
                    (1, 1),
                ],
                entries: &[
                    ("мин", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("сарын", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("сек", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("өдрийн", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("хоногийн", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("жилийн", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("цагийн", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("минут", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("секунд", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ц", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("долоо", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("минутын", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("секундын", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::M, DateCmp::D],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("өнөөдөр", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("өчигдөр", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("сая", 6),
                    ("тэрбум", 9),
                    ("мянга", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("үзэлтгүй", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("сингл", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("жүжиг", AlbumType::Show),
                    ("аудио ном", AlbumType::Audiobook),
                    ("цомог", AlbumType::Album),
                ],
            },
        },
        Language::Mr => Entry {
            timeago_tokens: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (1, 2),
                    (8, 0),
                    (13, 7),
                    (0, 4),
                ],
                entries: &[
                    ("स\u{947}क\u{902}दाप\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("वर\u{94d}षा\u{902}प\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("आठवड\u{94d}याप\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("स\u{947}क\u{902}द", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("मिनिटाप\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("स\u{947}क\u{902}दा\u{902}प\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("तासा\u{902}प\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("दिवसा\u{902}प\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("मिनिटा\u{902}प\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ता", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("तासाप\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("आठवड\u{94d}या\u{902}प\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("मिनिट", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("वर\u{94d}षाप\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("मिनि", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("दिवसाप\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("महिन\u{94d}याप\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("मिनिट\u{947}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("महिन\u{94d}या\u{902}प\u{942}र\u{94d}वी", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("स\u{947}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (2, 4),
                    (0, 0),
                    (10, 10),
                ],
                entries: &[
                    ("जान\u{947},", 1),
                    ("ऑक\u{94d}टो,", 10),
                    ("मार\u{94d}च,", 3),
                    ("ज\u{941}ल\u{948},", 7),
                    ("एप\u{94d}रि,", 4),
                    ("नोव\u{94d}ह\u{947}\u{902},", 11),
                    ("ज\u{942}न,", 6),
                    ("ऑग,", 8),
                    ("म\u{947},", 5),
                    ("सप\u{94d}ट\u{947}\u{902},", 9),
                    ("फ\u{947}ब\u{94d}र\u{941},", 2),
                    ("डिस\u{947}\u{902},", 12),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("आज", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("today", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("काल", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("ह", 3),
                    ("अब\u{94d}ज", 9),
                    ("लाख", 5),
                    ("कोटी", 7),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("नाहीत", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("भाग", AlbumType::Ep),
                    ("सि\u{902}गल", AlbumType::Single),
                    ("अल\u{94d}बम", AlbumType::Album),
                    ("ऑडिओब\u{941}क", AlbumType::Audiobook),
                    ("शो", AlbumType::Show),
                ],
            },
        },
        Language::Ms => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (3, 10),
                    (0, 0),
                    (10, 9),
                ],
                entries: &[
                    ("tahun", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("saat", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minit", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("mgu", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("jam", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("bln", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minggu", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("bulan", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("thn", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("hari", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 1),
                    (3, 3),
                    (0, 0),
                ],
                entries: &[
                    ("jan", 1),
                    ("sep", 9),
                    ("mac", 3),
                    ("jun", 6),
                    ("mei", 5),
                    ("nov", 11),
                    ("apr", 4),
                    ("feb", 2),
                    ("ogo", 8),
                    ("jul", 7),
                    ("dis", 12),
                    ("okt", 10),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("semalam", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ini", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("b", 9),
                    ("j", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("tiada", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("rekod single", AlbumType::Single),
                    ("album", AlbumType::Album),
                    ("buku audio", AlbumType::Audiobook),
                    ("rancangan", AlbumType::Show),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::My => Entry {
            timeago_tokens: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (0, 0),
                    (0, 2),
                ],
                entries: &[
                    ("လ", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sec", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("မ\u{102d}နစ\u{103a}န\u{103e}င\u{1037}\u{103a}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ရက\u{103a}", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ရက\u{103a}က", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("နာရ\u{102e}", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("စက\u{1039}ကန\u{1037}\u{103a}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("မ\u{102d}နစ\u{103a}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ပတ\u{103a}", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("န\u{103e}စ\u{103a}", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::D],
            months: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (1, 2),
                    (5, 4),
                    (0, 0),
                ],
                entries: &[
                    ("မေ", 5),
                    ("ဩ", 8),
                    ("ဇ\u{103d}န\u{103a}", 6),
                    ("ဒ\u{102e}", 12),
                    ("စက\u{103a}", 9),
                    ("မတ\u{103a}", 3),
                    ("ဇန\u{103a}", 1),
                    ("ဇ\u{1030}", 7),
                    ("အောက\u{103a}", 10),
                    ("န\u{102d}\u{102f}", 11),
                    ("ဖေ", 2),
                    ("ဧ", 4),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ယနေ\u{1037}", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("မနေ\u{1037}က", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (3, 0),
                    (0, 2),
                ],
                entries: &[
                    ("ထ", 3),
                    ("က\u{102f}ဋေ", 7),
                    ("သန\u{103a}း", 6),
                    ("ထောင\u{103a}", 3),
                    ("သ\u{102d}န\u{103a}း", 5),
                    ("သောင\u{103a}း", 4),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("၁", 1),
                    ("မရ\u{103e}\u{102d}", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("တစ\u{103a}က\u{102d}\u{102f}ယ\u{103a}တော\u{103a}", AlbumType::Single),
                    ("ရ\u{103e}\u{102d}\u{102f}း", AlbumType::Show),
                    ("အယ\u{103a}လ\u{103a}ဘမ\u{103a}", AlbumType::Album),
                    ("အော\u{103a}ဒ\u{102e}ယ\u{102d}\u{102f}စာအ\u{102f}ပ\u{103a}", AlbumType::Audiobook),
                ],
            },
        },
        Language::Ne => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (4, 0),
                    (1, 3),
                ],
                entries: &[
                    ("मिन\u{947}ट", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("वर\u{94d}ष", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("दिन", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("घण\u{94d}टा", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("दिनअघि", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("हप\u{94d}ता", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("महिना", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("स\u{947}क\u{947}न\u{94d}ड", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::D],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                    (7, 5),
                    (3, 0),
                ],
                entries: &[
                    ("अगस\u{94d}ट", 8),
                    ("ज\u{941}लाई", 7),
                    ("स\u{947}प\u{94d}ट\u{947}म\u{94d}बर", 9),
                    ("म\u{947}", 5),
                    ("नोभ\u{947}म\u{94d}बर", 11),
                    ("डिस\u{947}म\u{94d}बर", 12),
                    ("जनवरी", 1),
                    ("मार\u{94d}च", 3),
                    ("अप\u{94d}रिल", 4),
                    ("फ\u{947}ब\u{94d}र\u{941}अरी", 2),
                    ("ज\u{941}न", 6),
                    ("अक\u{94d}टोबर", 10),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("आज", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("हिजो", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("करोड", 7),
                    ("अरब", 9),
                    ("हजार", 3),
                    ("लाख", 5),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("छ\u{948}न", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 6581282999337146909,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("एकल एल\u{94d}बम", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("एल\u{94d}बम", AlbumType::Album),
                    ("टिभी सो", AlbumType::Show),
                    ("अडियोब\u{941}क", AlbumType::Audiobook),
                ],
            },
        },
        Language::Nl => Entry {
            timeago_tokens: ::phf::Map {
                key: 471159234146692604,
                disps: &[
                    (3, 0),
                    (12, 4),
                    (1, 8),
                ],
                entries: &[
                    ("sec", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("week", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("seconden", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("dgn", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("maanden", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minuten", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("maand", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("weken", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("uur", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minuut", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("dag", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("seconde", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("jaar", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("dagen", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (7, 0),
                    (0, 6),
                    (0, 0),
                ],
                entries: &[
                    ("feb", 2),
                    ("sep", 9),
                    ("mrt", 3),
                    ("jun", 6),
                    ("mei", 5),
                    ("nov", 11),
                    ("apr", 4),
                    ("aug", 8),
                    ("jul", 7),
                    ("dec", 12),
                    ("jan", 1),
                    ("okt", 10),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("gisteren", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("vandaag", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mld", 9),
                    ("mln", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("geen", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (4, 0),
                ],
                entries: &[
                    ("album", AlbumType::Album),
                    ("single", AlbumType::Single),
                    ("audioboek", AlbumType::Audiobook),
                    ("aflevering", AlbumType::Show),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::No => Entry {
            timeago_tokens: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (3, 5),
                    (1, 4),
                    (2, 18),
                    (3, 0),
                ],
                entries: &[
                    ("måned", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekund", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("år", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("md", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("dager", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("u", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("uke", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("uker", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("time", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("t", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dag", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("måneder", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minutt", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("døgn", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("timer", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minutter", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sekunder", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("d", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 1937371814602216758,
                disps: &[
                    (14, 0),
                    (10, 3),
                    (1, 8),
                ],
                entries: &[
                    ("jan", 1),
                    ("mar", 3),
                    ("apr", 4),
                    ("jul", 7),
                    ("des", 12),
                    ("juli", 7),
                    ("okt", 10),
                    ("mars", 3),
                    ("juni", 6),
                    ("jun", 6),
                    ("feb", 2),
                    ("aug", 8),
                    ("nov", 11),
                    ("sep", 9),
                    ("mai", 5),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("dag", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("går", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mrd", 9),
                    ("mill", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ingen", 0),
                    ("én", 1),
                ],
            },
            album_types: ::phf::Map {
                key: 12213676231523076107,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("singel", AlbumType::Single),
                    ("album", AlbumType::Album),
                    ("hørespill", AlbumType::Show),
                    ("lydbok", AlbumType::Audiobook),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::Or => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (6, 1),
                    (13, 8),
                    (1, 0),
                ],
                entries: &[
                    ("ବ", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ସପ\u{b4d}ତ\u{b3e}ହ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ମ\u{b3e}ସ", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ମ\u{b3f}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ମ\u{b3f}ନ\u{b3f}ଟ\u{b4d}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ମ\u{b3e}", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ଘଣ\u{b4d}ଟ\u{b3e}", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ଦ\u{b3f}ନ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ଘ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ସପ\u{b4d}ତ\u{b3e}", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ବର\u{b4d}ଷ", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ମ\u{b3f}ନ\u{b3f}ଟ\u{b4d}\u{200c}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ସେକେଣ\u{b4d}ଡ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ସେ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (3, 0),
                    (1, 0),
                    (10, 10),
                ],
                entries: &[
                    ("ସେପ\u{b4d}ଟେମ\u{b4d}ବର", 9),
                    ("ନଭେମ\u{b4d}ବର", 11),
                    ("ଜ\u{b41}ନ", 6),
                    ("ଅପ\u{b4d}ରେଲ", 4),
                    ("ଜ\u{b41}ଲ\u{b3e}ଇ", 7),
                    ("ଅକ\u{b4d}ଟୋବର", 10),
                    ("ମ\u{b3e}ର\u{b4d}ଚ\u{b4d}ଚ", 3),
                    ("ମଇ", 5),
                    ("ଜ\u{b3e}ନ\u{b41}ଆରୀ", 1),
                    ("ଅଗଷ\u{b4d}ଟ", 8),
                    ("ଡ\u{b3f}ସେମ\u{b4d}ବର", 12),
                    ("ଫେବ\u{b43}ଆରୀ", 2),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ଆଜ\u{b3f}", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ଗତକ\u{b3e}ଲ\u{b3f}", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (2, 0),
                    (2, 1),
                ],
                entries: &[
                    ("ନ\u{b3f}ଜଣ", 6),
                    ("ବ\u{b3f}", 9),
                    ("ନ\u{b3f}", 6),
                    ("ବ\u{b3f}ଜଣ", 9),
                    ("ବ\u{b3f}ଟ\u{b3f}", 9),
                    ("ହଜଣ", 3),
                    ("ନ\u{b3f}ଟ\u{b3f}", 6),
                    ("ହ", 3),
                    ("ହଟ\u{b3f}", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ନ\u{b3e}ହ\u{b3f}\u{b01}", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 2126027241312876569,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ଶୋ", AlbumType::Show),
                    ("ସ\u{b3f}ଙ\u{b4d}ଗଲ\u{b4d}", AlbumType::Single),
                    ("ଅଡ\u{b3f}ଓବ\u{b41}କ\u{b4d}", AlbumType::Audiobook),
                    ("ଆଲବମ\u{b4d}", AlbumType::Album),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::Pa => Entry {
            timeago_tokens: ::phf::Map {
                key: 399332969041170284,
                disps: &[
                    (0, 0),
                    (8, 1),
                ],
                entries: &[
                    ("ਹਫ\u{a3c}ਤ\u{a47}", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ਮਿ\u{a70}ਟ", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ਹਫ\u{a3c}ਤਾ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ਮਹੀਨਾ", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ਘ\u{a70}ਟ\u{a47}", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ਸਕਿ\u{a70}ਟ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ਸਾਲ", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ਮਹੀਨ\u{a47}", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ਦਿਨ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ਘ\u{a70}ਟਾ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (1, 0),
                    (0, 0),
                    (0, 6),
                ],
                entries: &[
                    ("ਮਾਰਚ", 3),
                    ("ਜ\u{a42}ਨ", 6),
                    ("ਜਨ", 1),
                    ("ਨਵ\u{a70}", 11),
                    ("ਦਸ\u{a70}", 12),
                    ("ਅਪ\u{a4d}ਰ\u{a48}", 4),
                    ("ਮਈ", 5),
                    ("ਜ\u{a41}ਲਾ", 7),
                    ("ਫ\u{a3c}ਰ", 2),
                    ("ਅਗ", 8),
                    ("ਸਤ\u{a70}", 9),
                    ("ਅਕਤ\u{a42}", 10),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ਅ\u{a71}ਜ", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ਕ\u{a71}ਲ\u{a4d}ਹ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12213676231523076107,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ਕਰ\u{a4b}ੜ", 7),
                    ("ਲ\u{a71}ਖ", 5),
                    ("ਹਜ\u{a3c}ਾਰ", 3),
                    ("ਅਰਬ", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ਨਹੀ\u{a02}", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ਆਡੀਓ-ਕਿਤਾਬ", AlbumType::Audiobook),
                    ("ਸਿ\u{a70}ਗਲ", AlbumType::Single),
                    ("ਐਲਬਮ", AlbumType::Album),
                    ("ep", AlbumType::Ep),
                    ("ਸ\u{a3c}\u{a4b}ਅ", AlbumType::Show),
                ],
            },
        },
        Language::Pl => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 7),
                    (0, 0),
                    (2, 1),
                    (0, 4),
                    (5, 9),
                    (0, 1),
                    (10, 12),
                ],
                entries: &[
                    ("tygodnia", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sekundy", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("godzin", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minutę", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("lat", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("lata", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("tydz", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sekund", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("rok", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("miesiąca", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minuta", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("godzinę", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dni", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("godziny", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("mies", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("tyg", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("miesiące", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("tydzień", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("tygodnie", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sekundę", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minuty", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minut", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("miesięcy", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("godz", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dzień", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("dnia", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sekunda", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("miesiąc", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("tygodni", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("roku", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 4),
                    (10, 8),
                    (2, 0),
                ],
                entries: &[
                    ("lis", 11),
                    ("sty", 1),
                    ("wrz", 9),
                    ("lip", 7),
                    ("lut", 2),
                    ("paź", 10),
                    ("cze", 6),
                    ("gru", 12),
                    ("sie", 8),
                    ("maj", 5),
                    ("kwi", 4),
                    ("mar", 3),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("dzisiaj", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("wczoraj", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("mln", 6),
                    ("mld", 9),
                    ("tys", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("brak", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 351906021642186605,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("album", AlbumType::Album),
                    ("słuchowisko", AlbumType::Show),
                    ("singiel", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("audiobook", AlbumType::Audiobook),
                ],
            },
        },
        Language::Pt => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                    (2, 2),
                    (8, 14),
                    (4, 10),
                ],
                entries: &[
                    ("horas", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("semanas", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("meses", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minuto", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("hora", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minutos", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("segundo", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("semana", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("anos", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("mês", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("segundos", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("dia", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sem", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("seg", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ano", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("dias", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                    (0, 3),
                    (3, 8),
                ],
                entries: &[
                    ("jan", 1),
                    ("out", 10),
                    ("nov", 11),
                    ("dez", 12),
                    ("set", 9),
                    ("jul", 7),
                    ("fev", 2),
                    ("mar", 3),
                    ("abr", 4),
                    ("mai", 5),
                    ("jun", 6),
                    ("ago", 8),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("hoje", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ontem", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("mi", 6),
                    ("bi", 9),
                    ("mil", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("um", 1),
                ],
            },
            album_types: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("álbum", AlbumType::Album),
                    ("ep", AlbumType::Ep),
                    ("single", AlbumType::Single),
                    ("programa", AlbumType::Show),
                    ("audiolivro", AlbumType::Audiobook),
                ],
            },
        },
        Language::PtPt => Entry {
            timeago_tokens: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (5, 9),
                    (3, 16),
                    (2, 0),
                    (4, 16),
                ],
                entries: &[
                    ("minutos", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("segundos", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("anos", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("mês", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("meses", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("semanas", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("semana", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("sem", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("segundo", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minuto", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("dias", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("horas", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ano", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("hora", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dia", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("hoje", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ontem", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mm", 9),
                    ("m", 6),
                    ("mil", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("álbum", AlbumType::Album),
                    ("single", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("programa", AlbumType::Show),
                    ("livro áudio", AlbumType::Audiobook),
                ],
            },
        },
        Language::Ro => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (11, 2),
                    (0, 5),
                    (2, 3),
                    (2, 0),
                ],
                entries: &[
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("zi", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("săptămână", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minut", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("oră", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("zile", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sec", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ore", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("an", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("secundă", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("săptămâni", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("lună", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("săpt", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("luni", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("minute", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("secunde", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ani", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (10, 3),
                    (1, 7),
                    (1, 0),
                ],
                entries: &[
                    ("iul", 7),
                    ("oct", 10),
                    ("apr", 4),
                    ("dec", 12),
                    ("mai", 5),
                    ("feb", 2),
                    ("mar", 3),
                    ("nov", 11),
                    ("sept", 9),
                    ("iun", 6),
                    ("aug", 8),
                    ("ian", 1),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("astăzi", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ieri", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mil", 6),
                    ("mld", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("un", 1),
                    ("nicio", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 7827257779606437632,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("emisiune", AlbumType::Show),
                    ("ep", AlbumType::Ep),
                    ("carte audio", AlbumType::Audiobook),
                    ("single", AlbumType::Single),
                    ("album", AlbumType::Album),
                ],
            },
        },
        Language::Ru => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 1),
                    (0, 0),
                    (5, 11),
                    (2, 28),
                    (0, 17),
                    (4, 5),
                    (1, 1),
                ],
                entries: &[
                    ("часов", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("час", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("минут", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("год", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("недель", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("сек", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("л", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("минуты", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("месяца", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("мес", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("г", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("секунду", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("секунды", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("дн", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("секунд", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("года", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("мин", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ч", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("месяцев", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("часа", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("месяц", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("дней", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("дня", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("неделю", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("с", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("день", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("лет", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("минуту", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("нед", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("недели", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("минута", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("секунда", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (1, 9),
                    (7, 3),
                    (1, 0),
                ],
                entries: &[
                    ("янв", 1),
                    ("сент", 9),
                    ("авг", 8),
                    ("июн", 6),
                    ("мая", 5),
                    ("апр", 4),
                    ("февр", 2),
                    ("дек", 12),
                    ("окт", 10),
                    ("июл", 7),
                    ("нояб", 11),
                    ("мар", 3),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("сегодня", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("вчера", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("млрд", 9),
                    ("млн", 6),
                    ("тыс", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            album_types: ::phf::Map {
                key: 345707026197253659,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("сингл", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("аудиокнига", AlbumType::Audiobook),
                    ("альбом", AlbumType::Album),
                    ("аудиошоу", AlbumType::Show),
                ],
            },
        },
        Language::Si => Entry {
            timeago_tokens: ::phf::Map {
                key: 471159234146692604,
                disps: &[
                    (2, 0),
                    (6, 0),
                ],
                entries: &[
                    ("පැය", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("තත\u{dca}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ම\u{dd2}න\u{dd2}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ම\u{dd2}න\u{dd2}ත\u{dca}ත\u{dd4}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("වසර", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ම\u{dcf}ස", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("සත\u{dd2}", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("තත\u{dca}පර", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ද\u{dd2}න", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::D],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 2),
                    (0, 0),
                    (0, 6),
                ],
                entries: &[
                    ("නොවැ", 11),
                    ("මැය\u{dd2}", 5),
                    ("සැප\u{dca}", 9),
                    ("අගෝ", 8),
                    ("ම\u{dcf}ර\u{dca}ත\u{dd4}", 3),
                    ("ජ\u{dd6}ල\u{dd2}", 7),
                    ("ඔක\u{dca}", 10),
                    ("පෙබ", 2),
                    ("ජන", 1),
                    ("දෙසැ", 12),
                    ("ජ\u{dd6}න\u{dd2}", 6),
                    ("අප\u{dca}\u{200d}රේල\u{dca}", 4),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("අද", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("ඊයේ", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ම\u{dd2}", 6),
                    ("බ\u{dd2}", 9),
                    ("ද", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("නැත", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 2126027241312876569,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("සංදර\u{dca}ශනය", AlbumType::Show),
                    ("ශ\u{dca}\u{200d}රව\u{dca}\u{200d}යපොත", AlbumType::Audiobook),
                    ("තන\u{dd2}", AlbumType::Single),
                    ("ද\u{dd3}ව\u{dcf}", AlbumType::Ep),
                    ("ඇල\u{dca}බමය", AlbumType::Album),
                ],
            },
        },
        Language::Sk => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 21),
                    (12, 8),
                    (25, 20),
                    (1, 2),
                    (0, 0),
                    (0, 10),
                    (1, 2),
                ],
                entries: &[
                    ("r", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sekúnd", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sekundami", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("dňom", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("rokmi", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("týž", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("týždňami", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("d", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("hodinami", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("mesiaca", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("mesiacmi", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("roka", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("minúta", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minútou", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minút", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("mesiacom", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("týždňom", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("hodinou", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dňami", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("rokom", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sekundy", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minúty", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minútami", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("mes", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("týždňa", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sekundou", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sekunda", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("dňa", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("včera", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("dnes", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("mld", 9),
                    ("tis", 3),
                    ("mil", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("žiadne", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 4066803471364472071,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("relácia", AlbumType::Show),
                    ("album", AlbumType::Album),
                    ("singel", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("audiokniha", AlbumType::Audiobook),
                ],
            },
        },
        Language::Sl => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 31),
                    (0, 5),
                    (2, 32),
                    (0, 0),
                    (0, 0),
                    (0, 30),
                    (0, 0),
                    (3, 12),
                ],
                entries: &[
                    ("dnevoma", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("minute", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sekundo", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minuta", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("tednom", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("dnevi", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("h", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("tednoma", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("mesecema", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekundami", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minuto", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minuti", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("tedni", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sekundi", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("urama", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("minut", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minutama", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minutami", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ted", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("mesecem", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekunde", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sekunda", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("dnevom", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sekund", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("leti", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("dnem", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("uro", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("letom", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("letoma", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("urami", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("sekundama", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("mes", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("meseci", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (3, 0),
                    (3, 4),
                    (0, 11),
                ],
                entries: &[
                    ("apr", 4),
                    ("jan", 1),
                    ("jul", 7),
                    ("jun", 6),
                    ("maj", 5),
                    ("mar", 3),
                    ("nov", 11),
                    ("feb", 2),
                    ("sep", 9),
                    ("dec", 12),
                    ("avg", 8),
                    ("okt", 10),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("danes", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("včeraj", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("mio", 6),
                    ("tis", 3),
                    ("mrd", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("brez", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (4, 0),
                ],
                entries: &[
                    ("album", AlbumType::Album),
                    ("singel", AlbumType::Single),
                    ("oddaja", AlbumType::Show),
                    ("zvočna knjiga", AlbumType::Audiobook),
                    ("ep", AlbumType::Ep),
                ],
            },
        },
        Language::Sq => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 7),
                    (3, 10),
                    (2, 0),
                ],
                entries: &[
                    ("muaj", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekonda", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("minuta", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("javë", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("vit", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("minutë", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sekondë", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("vjet", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ditë", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("orë", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (1, 0),
                    (1, 1),
                    (6, 6),
                ],
                entries: &[
                    ("shk", 2),
                    ("dhj", 12),
                    ("mar", 3),
                    ("sht", 9),
                    ("gush", 8),
                    ("pri", 4),
                    ("qer", 6),
                    ("korr", 7),
                    ("maj", 5),
                    ("tet", 10),
                    ("nën", 11),
                    ("jan", 1),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("dje", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sot", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("mln", 6),
                    ("mld", 9),
                    ("mijë", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("nuk", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 2980949210194914378,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("album", AlbumType::Album),
                    ("shfaq", AlbumType::Show),
                    ("ep", AlbumType::Ep),
                    ("libër me audio", AlbumType::Audiobook),
                    ("single", AlbumType::Single),
                ],
            },
        },
        Language::Sr => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 17),
                    (0, 2),
                    (0, 3),
                    (5, 0),
                    (2, 7),
                ],
                entries: &[
                    ("сати", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("минут", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("секунда", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ч", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("мес", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("месеци", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("секунди", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("недеља", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("година", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("мин", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("године", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("сата", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("сат", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("д", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("минута", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("месеца", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("недеље", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("год", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("секунде", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("нед", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("дана", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("сек", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("јуче", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("данас", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("млрд", 9),
                    ("мил", 6),
                    ("хиљ", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("нема", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 106375038446233661,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("аудио-књига", AlbumType::Audiobook),
                    ("ep", AlbumType::Ep),
                    ("сингл", AlbumType::Single),
                    ("албум", AlbumType::Album),
                    ("серија", AlbumType::Show),
                ],
            },
        },
        Language::SrLatn => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (5, 6),
                    (1, 0),
                    (1, 4),
                    (6, 10),
                    (4, 0),
                ],
                entries: &[
                    ("č", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("mes", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("god", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sekunde", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("dana", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("nedelje", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("godine", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("minuta", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sat", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("meseca", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekundi", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("nedelja", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("minut", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("d", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ned", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sekunda", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("sek", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("meseci", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sata", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("godina", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("sati", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("juče", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("danas", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("hilj", 3),
                    ("mlrd", 9),
                    ("mil", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("nema", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (4, 0),
                ],
                entries: &[
                    ("audio-knjiga", AlbumType::Audiobook),
                    ("singl", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("album", AlbumType::Album),
                    ("serija", AlbumType::Show),
                ],
            },
        },
        Language::Sv => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (13, 5),
                    (15, 0),
                    (6, 0),
                    (0, 17),
                ],
                entries: &[
                    ("år", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("d", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("veckor", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("timmar", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("månad", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("mån", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("vecka", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("sekunder", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("tim", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dag", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("dagar", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("minut", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("timme", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("min", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("v", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("s", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("minuter", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("månader", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekund", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                    (5, 5),
                    (4, 11),
                ],
                entries: &[
                    ("sep", 9),
                    ("jan", 1),
                    ("okt", 10),
                    ("dec", 12),
                    ("nov", 11),
                    ("apr", 4),
                    ("feb", 2),
                    ("mars", 3),
                    ("aug", 8),
                    ("juli", 7),
                    ("maj", 5),
                    ("juni", 6),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("dag", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("går", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("mn", 6),
                    ("md", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("inga", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("ljuddrama", AlbumType::Show),
                    ("ljudbok", AlbumType::Audiobook),
                    ("album", AlbumType::Album),
                    ("singel", AlbumType::Single),
                ],
            },
        },
        Language::Sw => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (3, 8),
                    (2, 0),
                ],
                entries: &[
                    ("mwaka", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("siku", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("wiki", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("saa", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("mwezi", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("sekunde", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("miezi", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("dakika", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("miaka", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (4, 0),
                    (1, 4),
                    (6, 1),
                ],
                entries: &[
                    ("jan", 1),
                    ("jul", 7),
                    ("mac", 3),
                    ("feb", 2),
                    ("nov", 11),
                    ("okt", 10),
                    ("apr", 4),
                    ("jun", 6),
                    ("sep", 9),
                    ("ago", 8),
                    ("des", 12),
                    ("mei", 5),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("leo", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("jana", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("m", 6),
                    ("elfu", 3),
                    ("b", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("haijatazamwa", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("singo", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("albamu", AlbumType::Album),
                    ("kitabu cha kusikiliza", AlbumType::Audiobook),
                    ("kipindi", AlbumType::Show),
                ],
            },
        },
        Language::Ta => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 11),
                    (11, 19),
                    (1, 12),
                    (2, 0),
                    (2, 12),
                ],
                entries: &[
                    ("ஆண\u{bcd}டுகளுக\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("விந\u{bbe}டிகள\u{bcd}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("வ\u{bbe}ரங\u{bcd}களுக\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("நிமிட", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ஆண\u{bcd}டிற\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ம\u{bbe}தத\u{bcd}துக\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("வ\u{bbe}ரத\u{bcd}திற\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("விந\u{bbe}டி", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("வ\u{bbe}ர", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("நிமிடம\u{bcd}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ம\u{bbe}தங\u{bcd}களுக\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("நிமிடங\u{bcd}களுக\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("விந\u{bbe}டிக\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("மணி", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("விந\u{bbe}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("மணிநேரம\u{bcd}", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("நிமிடங\u{bcd}கள\u{bcd}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ந\u{bbe}ளுக\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ந\u{bbe}ட\u{bcd}களுக\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("விந\u{bbe}டிகளுக\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ம\u{bbe}த", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("நிமி", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("நிமிடத\u{bcd}திற\u{bcd}கு", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 10),
                    (9, 8),
                    (3, 0),
                ],
                entries: &[
                    ("ஆக,", 8),
                    ("பிப\u{bcd},", 2),
                    ("ஏப\u{bcd},", 4),
                    ("அக\u{bcd},", 10),
                    ("டிச,", 12),
                    ("நவ,", 11),
                    ("ஜூலை,", 7),
                    ("ம\u{bbe}ர\u{bcd},", 3),
                    ("ஜன,", 1),
                    ("மே,", 5),
                    ("ஜூன\u{bcd},", 6),
                    ("செப\u{bcd},", 9),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("நேற\u{bcd}று", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("இன\u{bcd}று", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("லட\u{bcd}சம\u{bcd}", 5),
                    ("கோடி", 7),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("இல\u{bcd}லை", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 106375038446233661,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("சிங\u{bcd}கிள\u{bcd}", AlbumType::Single),
                    ("ஆல\u{bcd}பம\u{bcd}", AlbumType::Album),
                    ("ep", AlbumType::Ep),
                    ("ஆடியோ ஷோ", AlbumType::Show),
                    ("ஆடியோ புத\u{bcd}தகம\u{bcd}", AlbumType::Audiobook),
                ],
            },
        },
        Language::Te => Entry {
            timeago_tokens: ::phf::Map {
                key: 3599879742736855518,
                disps: &[
                    (4, 0),
                    (0, 0),
                    (8, 4),
                    (8, 5),
                ],
                entries: &[
                    ("స\u{c46}కన\u{c4d}ల", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("న\u{c3f}మ\u{c3f}ష\u{c3e}లు", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("న\u{c46}లల", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("న\u{c3f}మ\u{c3f}షం", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("వ\u{c3e}రం", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("గం", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("న\u{c3f}మ\u{c3f}ష\u{c3e}ల", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ర\u{c4b}జుల", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("స\u{c46}కన\u{c4d}లు", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("స\u{c46}కను", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ర\u{c4b}జు", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("సంవత\u{c4d}సరం", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("వ\u{c3e}ర\u{c3e}ల", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("గంట", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("గంటల", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("స\u{c46}క", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("న\u{c3f}మ\u{c3f}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("సం", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("న\u{c46}ల", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("సంవత\u{c4d}సర\u{c3e}ల", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (5, 6),
                    (0, 0),
                    (1, 0),
                ],
                entries: &[
                    ("ఏప\u{c4d}ర\u{c3f},", 4),
                    ("జన,", 1),
                    ("డ\u{c3f}స\u{c46}ం,", 12),
                    ("ఫ\u{c3f}బ\u{c4d}ర,", 2),
                    ("అక\u{c4d}ట\u{c4b},", 10),
                    ("నవం,", 11),
                    ("మ\u{c47},", 5),
                    ("జూన\u{c4d},", 6),
                    ("స\u{c46}ప\u{c4d}ట\u{c46}ం,", 9),
                    ("ఆగ,", 8),
                    ("జుల\u{c48},", 7),
                    ("మ\u{c3e}ర\u{c4d}చ\u{c3f},", 3),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("ఈ", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("న\u{c3f}న\u{c4d}న", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("క\u{c4b}ట\u{c4d}లు", 7),
                    ("లక\u{c4d}ష", 5),
                    ("లక\u{c4d}షలు", 5),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ల\u{c47}వు", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 345707026197253659,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("ఆడ\u{c3f}య\u{c4b}బుక\u{c4d}", AlbumType::Audiobook),
                    ("ఆల\u{c4d}బమ\u{c4d}", AlbumType::Album),
                    ("ep", AlbumType::Ep),
                    ("చూప\u{c3f}ంచు", AlbumType::Show),
                    ("స\u{c3f}ంగ\u{c3f}ల\u{c4d}", AlbumType::Single),
                ],
            },
        },
        Language::Th => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                    (2, 10),
                    (0, 11),
                    (8, 12),
                ],
                entries: &[
                    ("ว\u{e31}นท\u{e35}\u{e48}แล\u{e49}ว", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("นาท\u{e35}ท\u{e35}\u{e48}แล\u{e49}ว", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ว\u{e34}นาท\u{e35}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ว\u{e34}", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ท\u{e35}\u{e48}แล\u{e49}ว", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ป\u{e35}ท\u{e35}\u{e48}แล\u{e49}ว", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ช\u{e31}\u{e48}วโมงท\u{e35}\u{e48}ผ\u{e48}านมา", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("นาท\u{e35}", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ว\u{e31}นท\u{e35}\u{e48}ผ\u{e48}านมา", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("นาท\u{e35}ท\u{e35}\u{e48}ผ\u{e48}านมา", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ส\u{e31}ปดาห\u{e4c}ท\u{e35}\u{e48}ผ\u{e48}านมา", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ว\u{e34}นาท\u{e35}ท\u{e35}\u{e48}แล\u{e49}ว", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("เด\u{e37}อนท\u{e35}\u{e48}ผ\u{e48}านมา", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("ส\u{e31}ปดาห\u{e4c}ท\u{e35}\u{e48}แล\u{e49}ว", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("ชม", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ว\u{e34}นาท\u{e35}ท\u{e35}\u{e48}ผ\u{e48}านมา", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("เด\u{e37}อนท\u{e35}\u{e48}แล\u{e49}ว", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (2, 0),
                    (2, 0),
                    (0, 11),
                ],
                entries: &[
                    ("สค", 8),
                    ("มค", 1),
                    ("ตค", 10),
                    ("ม\u{e34}ย", 6),
                    ("เมย", 4),
                    ("ม\u{e35}ค", 3),
                    ("กพ", 2),
                    ("กย", 9),
                    ("กค", 7),
                    ("ธค", 12),
                    ("พค", 5),
                    ("พย", 11),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("อ\u{e31}ปเดตเม\u{e37}\u{e48}อวานน\u{e35}\u{e49}", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("อ\u{e31}ปเดตแล\u{e49}วว\u{e31}นน\u{e35}\u{e49}", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 1),
                    (4, 0),
                ],
                entries: &[
                    ("พ\u{e31}น", 3),
                    ("หม\u{e37}\u{e48}น", 4),
                    ("พ\u{e31}นล\u{e49}าน", 9),
                    ("แสน", 5),
                    ("หม\u{e37}\u{e48}นล\u{e49}าน", 10),
                    ("ล\u{e49}าน", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ไม\u{e48}ม\u{e35}การด\u{e39}", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 8694567506910003252,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("หน\u{e31}งส\u{e37}อเส\u{e35}ยง", AlbumType::Audiobook),
                    ("รายการ", AlbumType::Show),
                    ("ซ\u{e34}งเก\u{e34}ล", AlbumType::Single),
                    ("ep", AlbumType::Ep),
                    ("อ\u{e31}ลบ\u{e31}\u{e49}ม", AlbumType::Album),
                ],
            },
        },
        Language::Tr => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (4, 10),
                    (1, 0),
                    (1, 0),
                ],
                entries: &[
                    ("yıl", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("sn", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("hafta", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("hf", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("dk", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("saniye", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("saat", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("dakika", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("ay", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("gün", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sa", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 0),
                    (6, 9),
                    (0, 3),
                ],
                entries: &[
                    ("haz", 6),
                    ("ağu", 8),
                    ("oca", 1),
                    ("nis", 4),
                    ("eyl", 9),
                    ("mar", 3),
                    ("ara", 12),
                    ("eki", 10),
                    ("kas", 11),
                    ("şub", 2),
                    ("tem", 7),
                    ("may", 5),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("dün", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("bugün", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("b", 3),
                    ("mn", 6),
                    ("mr", 9),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("yok", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 4066803471364472071,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("sesli kitap", AlbumType::Audiobook),
                    ("single", AlbumType::Single),
                    ("program", AlbumType::Show),
                    ("ep", AlbumType::Ep),
                    ("albüm", AlbumType::Album),
                ],
            },
        },
        Language::Uk => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 14),
                    (1, 5),
                    (10, 33),
                    (1, 10),
                    (4, 9),
                    (1, 0),
                    (1, 24),
                ],
                entries: &[
                    ("днів", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("хвилини", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("секунда", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("години", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("тижня", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("тижні", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("дня", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("тиждень", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("секунду", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("день", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("тижнів", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("годину", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("року", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("місяців", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("годин", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("тиж", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("хв", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("р", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("секунд", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("місяця", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("хвилин", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("роки", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("міс", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("років", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("хвилину", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("рік", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("год", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("місяць", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("дн", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("дні", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("секунди", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("місяці", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("с", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("хвилина", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (3, 5),
                    (3, 0),
                    (0, 11),
                ],
                entries: &[
                    ("бер", 3),
                    ("лют", 2),
                    ("жовт", 10),
                    ("квіт", 4),
                    ("серп", 8),
                    ("вер", 9),
                    ("груд", 12),
                    ("черв", 6),
                    ("лип", 7),
                    ("січ", 1),
                    ("лист", 11),
                    ("трав", 5),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("вчора", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("сьогодні", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("млрд", 9),
                    ("тис", 3),
                    ("млн", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("жодного", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("сингл", AlbumType::Single),
                    ("альбом", AlbumType::Album),
                    ("аудіокнига", AlbumType::Audiobook),
                    ("мініальбом", AlbumType::Ep),
                    ("аудіодрама", AlbumType::Show),
                ],
            },
        },
        Language::Ur => Entry {
            timeago_tokens: ::phf::Map {
                key: 4066803471364472071,
                disps: &[
                    (14, 4),
                    (2, 0),
                    (1, 6),
                ],
                entries: &[
                    ("ہفتہ", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("مہینے", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("سال", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("گھنٹے", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("منٹ،", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("گھنٹہ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("ماہ", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("دن", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ہفتے", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("مہینہ", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("منٹس", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("منٹ", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("سیکنڈ", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("دنوں", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("سیکنڈز", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                    (10, 7),
                    (8, 7),
                ],
                entries: &[
                    ("فروری،", 2),
                    ("اکتوبر،", 10),
                    ("جنوری،", 1),
                    ("اپریل،", 4),
                    ("ستمبر،", 9),
                    ("دسمبر،", 12),
                    ("نومبر،", 11),
                    ("مارچ،", 3),
                    ("اگست،", 8),
                    ("جون،", 6),
                    ("جولائی،", 7),
                    ("مئی،", 5),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("آج", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("کل", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ہزار", 3),
                    ("ارب", 9),
                    ("کروڑ", 7),
                    ("لاکھ", 5),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("نہیں", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 106375038446233661,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("آڈیو بک", AlbumType::Audiobook),
                    ("البم", AlbumType::Album),
                    ("ep", AlbumType::Ep),
                    ("واحد", AlbumType::Single),
                    ("شو", AlbumType::Show),
                ],
            },
        },
        Language::Uz => Entry {
            timeago_tokens: ::phf::Map {
                key: 351906021642186605,
                disps: &[
                    (2, 4),
                    (1, 0),
                ],
                entries: &[
                    ("soat", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("oy", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("yil", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("soniya", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("daqiqa", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("hafta", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("kun", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("daq", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("son", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 5),
                    (0, 0),
                    (9, 6),
                ],
                entries: &[
                    ("mar,", 3),
                    ("may,", 5),
                    ("noy,", 11),
                    ("avg,", 8),
                    ("iyl,", 7),
                    ("iyn,", 6),
                    ("yan,", 1),
                    ("fev,", 2),
                    ("dek,", 12),
                    ("okt,", 10),
                    ("sen,", 9),
                    ("apr,", 4),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("kecha", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("bugun", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("mln", 6),
                    ("mlrd", 9),
                    ("ming", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            album_types: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("shou", AlbumType::Show),
                    ("ep", AlbumType::Ep),
                    ("albom", AlbumType::Album),
                    ("singl", AlbumType::Single),
                    ("audiokitob", AlbumType::Audiobook),
                ],
            },
        },
        Language::Vi => Entry {
            timeago_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (2, 3),
                    (1, 0),
                ],
                entries: &[
                    ("giờ", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("giây", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("ngày", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("tháng", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("phút", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("năm", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("tuần", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::M, DateCmp::Y],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("qua", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("nay", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: true,
            number_tokens: ::phf::Map {
                key: 14108922650502679131,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("tr", 6),
                    ("t", 9),
                    ("n", 3),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            album_types: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("đĩa đơn", AlbumType::Single),
                    ("đĩa nhạc mở rộng (ep)", AlbumType::Ep),
                    ("chương trình", AlbumType::Show),
                    ("sách nói", AlbumType::Audiobook),
                    ("đĩa nhạc", AlbumType::Album),
                ],
            },
        },
        Language::ZhCn => Entry {
            timeago_tokens: ::phf::Map {
                key: 2980949210194914378,
                disps: &[
                    (0, 0),
                    (2, 1),
                ],
                entries: &[
                    ("分", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("秒", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("年", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("周", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("月", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("天", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("小", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::M, DateCmp::D],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("今", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("昨", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("万", 4),
                    ("亿", 8),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("无", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (3, 0),
                ],
                entries: &[
                    ("迷你专辑", AlbumType::Ep),
                    ("有声读物", AlbumType::Audiobook),
                    ("专辑", AlbumType::Album),
                    ("广播剧", AlbumType::Show),
                    ("单曲", AlbumType::Single),
                ],
            },
        },
        Language::ZhHk => Entry {
            timeago_tokens: ::phf::Map {
                key: 12213676231523076107,
                disps: &[
                    (3, 3),
                    (2, 0),
                ],
                entries: &[
                    ("月", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("天", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("小", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("期", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("年", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("秒", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("日", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("分", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::M, DateCmp::D],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("今", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("昨", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("b", 9),
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            album_types: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("有聲書", AlbumType::Audiobook),
                    ("專輯", AlbumType::Album),
                    ("節目", AlbumType::Show),
                    ("單曲", AlbumType::Single),
                ],
            },
        },
        Language::ZhTw => Entry {
            timeago_tokens: ::phf::Map {
                key: 10121458955350035957,
                disps: &[
                    (5, 0),
                    (6, 5),
                ],
                entries: &[
                    ("天", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("年", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("秒", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("週", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("小", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("月", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("分", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                ],
            },
            date_order: &[DateCmp::Y, DateCmp::M, DateCmp::D],
            months: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (1, 0),
                ],
                entries: &[
                    ("今", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                    ("昨", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("萬", 4),
                    ("億", 8),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                ],
                entries: &[
                ],
            },
            album_types: ::phf::Map {
                key: 15467950696543387533,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("ep", AlbumType::Ep),
                    ("有聲書", AlbumType::Audiobook),
                    ("專輯", AlbumType::Album),
                    ("節目", AlbumType::Show),
                    ("單曲", AlbumType::Single),
                ],
            },
        },
        Language::Zu => Entry {
            timeago_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (4, 21),
                    (12, 15),
                    (0, 14),
                    (2, 0),
                    (16, 18),
                ],
                entries: &[
                    ("izinyanga", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("iminithi", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("iviki", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("amaminithi", TaToken { n: 1, unit: Some(TimeUnit::Minute) }),
                    ("osukwini", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ihora", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                    ("usuku", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("sec", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("odlule", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("ezedlule", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("inyanga", TaToken { n: 1, unit: Some(TimeUnit::Month) }),
                    ("iminyaka", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("izinsuku", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ezingu", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("amasekhondi", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("olwedlule", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("ezinsukwini", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("evikini", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("unyaka", TaToken { n: 1, unit: Some(TimeUnit::Year) }),
                    ("amaviki", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("olungu", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("elingu", TaToken { n: 1, unit: Some(TimeUnit::Week) }),
                    ("isekhondi", TaToken { n: 1, unit: Some(TimeUnit::Second) }),
                    ("emahoreni", TaToken { n: 1, unit: Some(TimeUnit::Hour) }),
                ],
            },
            date_order: &[DateCmp::D, DateCmp::Y],
            months: ::phf::Map {
                key: 7485420634051515786,
                disps: &[
                    (2, 0),
                    (1, 9),
                    (1, 10),
                ],
                entries: &[
                    ("dis", 12),
                    ("jul", 7),
                    ("aga", 8),
                    ("okt", 10),
                    ("feb", 2),
                    ("nov", 11),
                    ("sep", 9),
                    ("mas", 3),
                    ("eph", 4),
                    ("mey", 5),
                    ("jan", 1),
                    ("jun", 6),
                ],
            },
            timeago_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("izolo", TaToken { n: 1, unit: Some(TimeUnit::Day) }),
                    ("namuhla", TaToken { n: 0, unit: Some(TimeUnit::Day) }),
                ],
            },
            comma_decimal: false,
            number_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("b", 9),
                    ("m", 6),
                ],
            },
            number_nd_tokens: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (0, 0),
                ],
                entries: &[
                    ("akukho", 0),
                ],
            },
            album_types: ::phf::Map {
                key: 12913932095322966823,
                disps: &[
                    (2, 0),
                ],
                entries: &[
                    ("i-albhamu", AlbumType::Album),
                    ("i-audiobook", AlbumType::Audiobook),
                    ("i-single", AlbumType::Single),
                    ("bonisa", AlbumType::Show),
                    ("i-ep", AlbumType::Ep),
                ],
            },
        },
    }
}
